/*
SUPPORT: sinass Sharma, Ashish
*/
/******************************************************************************
 * Job:             Load_STG2DM_Pack_Inventory_fact                           *
 * Description:                                                               *
 *                                                                            *
 *                                                                            *
 * Source Tables:   GEOGRAPHY_DM - DI_DM.GEOGRAPHY_DM                         *
 *                  PRODUCT_DM - DI_DM.PRODUCT_DM                             *
 *                  STG_INVENTORY - DIDM_STG.STG_INVENTORY                    *
 * Target Table:    PACK_INVENTORY_FACT -                                     *
 *                   DI_DM.PACK_INVENTORY_FACT                                *
 *                                                                            *
 * Generated on:    Wednesday, November 12, 2008 11:50:35 AM IST              *
 * Generated by:    sinsoj@row                                                *
 * Version:         SAS Data Integration Studio 4.2                           *
 ******************************************************************************/

/* General macro variables  */
%let jobID = %quote(A5RCIHUB.BA0001XR);
%let etls_jobName = %nrquote(Load_STG2DM_Pack_Inventory_fact);
%let etls_userID = %nrquote(sinsoj@row);

/* Setup to capture return codes  */
%global job_rc trans_rc sqlrc;
%let sysrc = 0;
%let job_rc = 0;
%let trans_rc = 0;
%let sqlrc = 0;
%global syserr;

%macro rcSet(error);
   %if (&error gt &trans_rc) %then
      %let trans_rc = &error;
   %if (&error gt &job_rc) %then
      %let job_rc = &error;
%mend rcSet;

%macro rcSetDS(error);
   if &error gt input(symget('trans_rc'),12.) then
      call symput('trans_rc',trim(left(put(&error,12.))));
   if &error gt input(symget('job_rc'),12.) then
      call symput('job_rc',trim(left(put(&error,12.))));
%mend rcSetDS;

/* Create metadata macro variables */
%let IOMServer      = %nrquote(MIMain);

/* Setup for capturing job status  */
%let etls_startTime = %sysfunc(datetime(),datetime.);
%let etls_recordsBefore = 0;
%let etls_recordsAfter = 0;
%let etls_lib = 0;
%let etls_table = 0;

%global etls_debug;
%macro etls_setDebug;
   %if %str(&etls_debug) ne 0 %then
      OPTIONS MPRINT%str(;);
%mend;
%etls_setDebug;

/*---- Start of Pre-Process Code  ----*/

%etl_initjobparam (monlib=di_mon, jobname=%nrquote(&etls_jobName));
/**/
%etl_initchangeflags;
/*---- End of Pre-Process Code  ----*/

%rcSet(&syserr);
%rcSet(&sqlrc);

/*============================================================================*
 * Step:            UW-Check SZPK Application                                 *
 * Transform:       User Written                                              *
 * Description:                                                               *
 *                                                                            *
 * Source Table:    STG_INVENTORY - DIDM_STG.STG_INVENTORY                    *
 * Target Table:    UW - DIDM_STG.STG_INVENTORY                               *
 *                                                                            *
 * User Written:    Get_lvl                                                   *
 *============================================================================*/

%let transformID = %quote(A5RCIHUB.BC0009QG);
%let trans_rc = 0;
%let etls_stepStartTime = %sysfunc(datetime(), datetime20.);
%let SYSLAST = %nrquote(DIDM_STG.STG_INVENTORY);

/*---- Start of User Written Code  ----*/

%global g_szpk_active_app_flg g_geo_prod_low_data_lvl_size g_szpk_prod_lvl
        g_szpk_geo_lvl g_curr_start_dt if_condition;

%let if_condition = %str(1=1);

%macro check_szpk_app;
/*Get ACTIVE_APP_FLG for SZPK application*/
PROC SQL noprint;
   select spec_value into :g_szpk_active_app_flg
   from di_dm.global_setting
   where upcase(spec_nm)='ACTIVE_APP_FLG'
   and upcase(app)='SZPK';
QUIT;
%rcSet(&sqlrc);

/* If Pack application is not set then stop running job */
%if &g_szpk_active_app_flg ne 1 %then
%do;
    %let etl_msg = %etl_msg(mid=5012,args=ACTIVE_APP_FLG(SZPK) GLOBAL_SETTING);
    %put &etl_msg;
    %etl_abort;
%end;

/*Get GEO_PROD_LOW_DATA_LVL_SIZE for size from global settings*/
PROC SQL noprint;
   select spec_value into :g_geo_prod_low_data_lvl_size
   from di_dm.global_setting
   where upcase(spec_nm)='GEO_PROD_LOW_DATA_LVL_SIZE';
QUIT;

%let g_geo_prod_low_data_lvl_size=&g_geo_prod_low_data_lvl_size;

/* validation for geo_prod_low_data_lvl_size */
%if &g_szpk_active_app_flg eq 1 and &g_geo_prod_low_data_lvl_size eq %str() %then
%do;
   %let etl_msg = %etl_msg(mid=5012,args=GEO_PROD_LOW_DATA_LVL_SIZE GLOBAL_SETTING);
   %put &etl_msg;
   %etl_abort;
%end;
%else
   %put g_geo_prod_low_data_lvl_size=&g_geo_prod_low_data_lvl_size;

/* Get Prod and Geo levels for SZPK */
PROC SQL noprint;
   select prod_lvl, geo_lvl into :g_szpk_prod_lvl, :g_szpk_geo_lvl
   from di_dm.data_lvl
   where data_lvl_id=&g_geo_prod_low_data_lvl_size and PROD_HIER_ASSOC_CD=1 and GEO_HIER_ASSOC_CD=1;
QUIT;

%let g_szpk_prod_lvl=&g_szpk_prod_lvl;
%let g_szpk_geo_lvl=&g_szpk_geo_lvl;

%if &sqlobs eq 0 %then
%do;
   %let etl_msg = %etl_msg(mid=5012,args=GEO_PROD_LOW_DATA_LVL_SIZE DATA_LVL);
   %put &etl_msg;
   %etl_abort;
%end;
%else
%do;
   %put Pack prod_lvl=&g_szpk_prod_lvl;
   %put Pack geo_lvl=&g_szpk_geo_lvl;
%end;

/* Identify the latest week from stg_inventory_pack */
%local current_date;
data _null_;
   call symput('current_date',put(today(),date9.));
run;

proc sql noprint;
   select max(start_dt) format=date9.
   into :g_curr_start_dt
   from didm_stg.stg_inventory
   where start_dt le "&current_date"d;
quit;

%put Current Week Start Date=&g_curr_start_dt;

%if &g_curr_start_dt eq %str(.) or &sqlobs eq 0 %then
   %do;
      %put WARNING: No Data in Stage for current week;
   %end;
%mend;

%check_szpk_app;
/*---- End of User Written Code  ----*/

%rcSet(&syserr);
%rcSet(&sqlrc);


/**  Step end UW-Check SZPK Application **/

/*============================================================================*
 * Step:            SAS Extract Current data                                  *
 * Transform:       Extract                                                   *
 * Description:                                                               *
 *                                                                            *
 * Source Table:    UW - DIDM_STG.STG_INVENTORY                               *
 * Target Table:    Extract Target -                                          *
 *                   work.tmp_stg_inv_pack_curr                               *
 *============================================================================*/

%let transformID = %quote(A5RCIHUB.BC0009QH);
%let trans_rc = 0;
%let etls_stepStartTime = %sysfunc(datetime(), datetime20.);

%let SYSLAST = %nrquote(DIDM_STG.STG_INVENTORY);

/*---- Map the columns  ----*/
proc datasets lib = work nolist nowarn memtype = (data view);
   delete tmp_stg_inv_pack_curr;
quit;

%put %str(NOTE: Mapping columns ...);
proc sql;
   create view work.tmp_stg_inv_pack_curr as
      select
         GEO_ID,
         PROD_ID,
         START_DT,
         END_DT,
         CLOSE_INV_UNITS,
         CLOSE_INV_AMT_AT_RETAIL,
         CLOSE_INV_AMT_AT_COST,
         CHANGE_FLG
   from &SYSLAST
      where (&if_condition) AND START_DT >= "&g_curr_start_dt"d
   ;
quit;

%let SYSLAST = work.tmp_stg_inv_pack_curr;

%rcSet(&sqlrc);

/**  Step end SAS Extract Current data **/

/*============================================================================*
 * Step:            SAS Sort                                                  *
 * Transform:       Sort                                                      *
 * Description:                                                               *
 *                                                                            *
 * Source Table:    Extract Target -                                          *
 *                   work.tmp_stg_inv_pack_curr                               *
 * Target Table:    Sort Target - work.stg_inv_pack_sorted                    *
 *============================================================================*/

%let transformID = %quote(A5RCIHUB.BC0009QI);
%let trans_rc = 0;
%let etls_stepStartTime = %sysfunc(datetime(), datetime20.);

%let SYSLAST = %nrquote(work.tmp_stg_inv_pack_curr);

%let ETLS_SYSLAST = &SYSLAST;
/*---- Start of Pre-Process Code  ----*/

%di_etl_keep_work_table(tmp_stg_inv_pack_curr);
/*---- End of Pre-Process Code  ----*/

%rcSet(&syserr);
%rcSet(&sqlrc);

%let SYSLAST = &ETLS_SYSLAST;

proc datasets lib = work nolist nowarn memtype = (data view);
   delete stg_inv_pack_sorted;
quit;

proc sort data = &SYSLAST
   out = work.stg_inv_pack_sorted;
   by
      GEO_ID
      PROD_ID
      START_DT
   ;
run;

%rcSet(&syserr);



/**  Step end SAS Sort **/

/*============================================================================*
 * Step:            UW-Identify duplicates                                    *
 * Transform:       User Written                                              *
 * Description:                                                               *
 *                                                                            *
 * Source Table:    Sort Target - work.stg_inv_pack_sorted                    *
 * Target Table:    UW-Code Target - work.tmp_stg_inv_pack                    *
 *                                                                            *
 * User Written:    Identify duplicates                                       *
 *============================================================================*/

%let transformID = %quote(A5RCIHUB.BC0009QJ);
%let trans_rc = 0;
%let etls_stepStartTime = %sysfunc(datetime(), datetime20.);

%let SYSLAST = %nrquote(work.stg_inv_pack_sorted);

%let ETLS_SYSLAST = &SYSLAST;
/*---- Start of Pre-Process Code  ----*/

%di_etl_keep_work_table(stg_inv_pack_sorted);
/*---- End of Pre-Process Code  ----*/

%rcSet(&syserr);
%rcSet(&sqlrc);

%let SYSLAST = &ETLS_SYSLAST;

%let _OUTPUT = work.tmp_stg_inv_pack;


/*---- Start of User Written Code  ----*/

Data &_output;
   set &SYSLAST;
   by geo_id prod_id start_dt;
   length ETLS_EXCEPTION_COND $180;
   if not last.start_dt then
   ETLS_EXCEPTION_COND = "%etl_msg(mid=5351)";
run;
/*---- End of User Written Code  ----*/

%rcSet(&syserr);
%rcSet(&sqlrc);


/**  Step end UW-Identify duplicates **/

/*============================================================================*
 * Step:            Extract Size lvl Data                                     *
 * Transform:       Extract                                                   *
 * Description:                                                               *
 *                                                                            *
 * Source Table:    PRODUCT_DM - DI_DM.PRODUCT_DM                             *
 * Target Table:    Extract Target - work.szpk_prod_dm                        *
 *============================================================================*/

%let transformID = %quote(A5RCIHUB.BC0009QK);
%let trans_rc = 0;
%let etls_stepStartTime = %sysfunc(datetime(), datetime20.);

%let SYSLAST = %nrquote(DI_DM.PRODUCT_DM);

/*---- Map the columns  ----*/
proc datasets lib = work nolist nowarn memtype = (data view);
   delete szpk_prod_dm;
quit;

%put %str(NOTE: Mapping columns ...);
proc sql;
   create view work.szpk_prod_dm as
      select
         (max(PROD_HIER_SK)) as PROD_HIER_SK length = 6,
         PROD_ID
   from &SYSLAST
      where (STD_HIER_PROD_LVL = &g_szpk_prod_lvl)
   group by
      PROD_ID
   ;
quit;

%let SYSLAST = work.szpk_prod_dm;

%rcSet(&sqlrc);



/**  Step end Extract Size lvl Data **/

/*============================================================================*
 * Step:            Lk_pack_inventory_fact                                    *
 * Transform:       Lookup                                                    *
 * Description:                                                               *
 *                                                                            *
 * Source Tables:   UW-Code Target - work.tmp_stg_inv_pack                    *
 *                  GEOGRAPHY_DM - DI_DM.GEOGRAPHY_DM                         *
 *                  Extract Target - work.szpk_prod_dm                        *
 * Target Table:    Lookup Target - work.tmp_inv_pack_curr                    *
 *============================================================================*/

%let transformID = %quote(A5RCIHUB.BC0009QL);
%let trans_rc = 0;
%let etls_stepStartTime = %sysfunc(datetime(), datetime20.);

%let ETLS_SYSLAST = &SYSLAST;
/*---- Start of Pre-Process Code  ----*/

%di_etl_keep_work_table(tmp_stg_inv_pack szpk_prod_dm);
/*---- End of Pre-Process Code  ----*/

%rcSet(&syserr);
%rcSet(&sqlrc);

%let SYSLAST = &ETLS_SYSLAST;

%let runTime = %sysfunc(datetime());

proc datasets lib = work nolist nowarn memtype = (data view);
   delete etls_temp_lookupview0;
quit;

proc sql; create view etls_temp_lookupview0 as
   select * from DI_DM.GEOGRAPHY_DM
   where (STD_HIER_GEO_LVL   = &g_szpk_GEO_LVL);
quit;
proc datasets lib = work nolist nowarn memtype = (data view);
   delete tmp_inv_pack_curr;
quit;

/*---- Begin DATA step to perform lookups  ----*/
DATA  work.tmp_inv_pack_curr
         ( keep = GEO_ID PROD_ID START_DT END_DT CLOSE_INV_UNITS CLOSE_INV_AMT_AT_RETAIL
                 CLOSE_INV_AMT_AT_COST CHANGE_FLG GEO_HIER_SK PROD_HIER_SK
                 ETLS_EXCEPTION_COND )
      ;

   attrib GEO_ID length=$32
          PROD_ID length=$32
          START_DT length=8 format=DATE9.
          END_DT length=8 format=DATE9.
          CLOSE_INV_UNITS length=8 format=NLNUM12.2
          CLOSE_INV_AMT_AT_RETAIL length=8
          CLOSE_INV_AMT_AT_COST length=8
          CHANGE_FLG length=$1
          GEO_HIER_SK length=4
          PROD_HIER_SK length=6
          ETLS_EXCEPTION_COND length=$180
          ;

   retain missing0-missing1 0;

   etls_assign_target_value = 0;
   /* Build hash objects from lookup tables before reading first source row  */
   if (_n_ = 1) then
   do;
      /* Build hash h0 from lookup table etls_temp_lookupview0 */
      nlobs = .;
      dsid = open("etls_temp_lookupview0");
      if (dsid > 0) then
      do;
         if ( attrc(dsid, 'MTYPE') = 'DATA' ) then
            nlobs = attrn(dsid, 'NLOBS');
         else
            nlobs = -1;

         dsid = close(dsid);
         if (nlobs ^= 0) then
         do;
            if (nlobs > 0) then
               exponent = ceil(log2(nlobs));
            else
               exponent = 8;
            declare hash h0(dataset: "etls_temp_lookupview0", hashexp: exponent);
            h0.defineKey( "GEO_ID");
            h0.defineData( "GEO_HIER_SK");
            h0.defineDone();
            if (nlobs = -1) then
            do;
               if (h0.Num_Items < 1) then
               do;
                  put "NOTE: Lookup table is empty: etls_temp_lookupview0";
                  missing0 = 98;
               end;
            end;
         end;
         else
         do;
            put "NOTE: Lookup table is empty: etls_temp_lookupview0";
            missing0 = 98;
         end;
      end;
      else
      do;
         put "NOTE: Lookup table does not exist or cannot be opened:"
              " etls_temp_lookupview0";

         put "NOTE: Abort action indicated, condition= Lookup table missing:"
              " etls_temp_lookupview0";

         abort 3;
      end;

      /* Build hash h1 from lookup table work.szpk_prod_dm */
      nlobs = .;
      dsid = open("work.szpk_prod_dm");
      if (dsid > 0) then
      do;
         if ( attrc(dsid, 'MTYPE') = 'DATA' ) then
            nlobs = attrn(dsid, 'NLOBS');
         else
            nlobs = -1;

         dsid = close(dsid);
         if (nlobs ^= 0) then
         do;
            if (nlobs > 0) then
               exponent = ceil(log2(nlobs));
            else
               exponent = 8;
            declare hash h1(dataset: "work.szpk_prod_dm", hashexp: exponent);
            h1.defineKey( "PROD_ID");
            h1.defineData( "PROD_HIER_SK");
            h1.defineDone();
            if (nlobs = -1) then
            do;
               if (h1.Num_Items < 1) then
               do;
                  put "NOTE: Lookup table is empty: work.szpk_prod_dm";
                  missing1 = 98;
               end;
            end;
         end;
         else
         do;
            put "NOTE: Lookup table is empty: work.szpk_prod_dm";
            missing1 = 98;
         end;
      end;
      else
      do;
         put "NOTE: Lookup table does not exist or cannot be opened:"
              " work.szpk_prod_dm";

         put "NOTE: Abort action indicated, condition= Lookup table missing:"
              " work.szpk_prod_dm";

         abort 3;
      end;

      call missing (GEO_ID, GEO_HIER_SK, PROD_ID, PROD_HIER_SK);

   end; /* All hash objects have been defined */

   /* Read a row from the source table  */
   set work.tmp_stg_inv_pack end = eof;

   /* Is the current key value stored in hash h0?  */
   if (missing0 < 1) then
   do;
      rc0 = h0.find();
   end;
   else
      rc0 = missing0;

   /* Is the current key value stored in hash h1?  */
   if (missing1 < 1) then
   do;
      rc1 = h1.find();
   end;
   else
      rc1 = missing1;

   /* Examine success of lookups  */
   if ( rc0=0 and rc1=0 ) then
   do;
      /* Write row to target  */
      output work.tmp_inv_pack_curr;
      return;
   end;
   else
   do;
      error_total + 1;

      if (rc0 ^= 0) then
      do;
         exception_total + 1;
         if (rc0 = 98) then
         do;
            /* Check: Lookup table has no rows-Write exception row  */
            /* Check: Lookup table has no rows-Set target columns to value  */
         end;

         else
         do;
            /* Check: Lookup value not found-Set target columns to value  */
            ETLS_EXCEPTION_COND = put(cats(ETLS_EXCEPTION_COND,'0a'x,"%etl_msg(mid=5305,args=GEO_ID)"),$180.);
            etls_assign_target_value = 1;
            /* Check: Lookup value not found-Set target columns to missing  */
            call missing (GEO_HIER_SK);
            etls_assign_target_value = 1;
         end;
      end;

      if (rc1 ^= 0) then
      do;
         exception_total + 1;
         if (rc1 = 98) then
         do;
            /* Check: Lookup table has no rows-Write exception row  */
            /* Check: Lookup table has no rows-Set target columns to value  */
         end;

         else
         do;
            /* Check: Lookup value not found-Set target columns to value  */
            ETLS_EXCEPTION_COND =put(cats(ETLS_EXCEPTION_COND,'0a'x,"%etl_msg(mid=5305,args=PROD_ID)"),$180.);
            etls_assign_target_value = 1;
            /* Check: Lookup value not found-Set target columns to missing  */
            call missing (PROD_HIER_SK);
            etls_assign_target_value = 1;
         end;
      end;

      /* Set target columns to value/missing requested?  */
      if (etls_assign_target_value eq 1) then
         /* Write row to target  */
         output work.tmp_inv_pack_curr;

   end; /* One or more lookups failed */

   if (eof = 1) then
   do;
      put "Source records with errors: " error_total ;
      put "Total lookup exceptions: " exception_total ;
   end;

run;

%rcSet(&syserr);



/**  Step end Lk_pack_inventory_fact **/

/*============================================================================*
 * Step:            Val_pack_inventory_fact                                   *
 * Transform:       Data Validation                                           *
 * Description:                                                               *
 *                                                                            *
 * Source Table:    Lookup Target - work.tmp_inv_pack_curr                    *
 * Target Table:    Data Validation Transform -                               *
 *                   work.valid_pack_inventory                                *
 *============================================================================*/

%let transformID = %quote(A5RCIHUB.BC0009QM);
%let trans_rc = 0;
%let etls_stepStartTime = %sysfunc(datetime(), datetime20.);

%let SYSLAST = %nrquote(work.tmp_inv_pack_curr);

%let ETLS_SYSLAST = &SYSLAST;
/*---- Start of Pre-Process Code  ----*/

%di_etl_keep_work_table(tmp_inv_pack_curr);
/*---- End of Pre-Process Code  ----*/

%rcSet(&syserr);
%rcSet(&sqlrc);

%let SYSLAST = &ETLS_SYSLAST;

/* System options  */
options ;
%let last=&syslast;
proc sql;
create view work.etls_errors_vw as select * from work.etls_errors(drop = PROD_HIER_SK GEO_HIER_SK);
quit;
%let syslast=&last;

%put %str(NOTE: Cannot create exception report because a view has been selected as output.);

%macro etls_Validate;

   %let runTime = %sysfunc(datetime());

   /* Create the Exception Report  */
   %macro makeReport;

      %if (&m_excp eq 1) %then
      %do;
         options nodate;

         proc sort data = work.etls_Exceptions;
            by x_row;
         run;

         proc print data = work.etls_Exceptions noobs label;
            title1 "Exceptions Found in Table: work.tmp_inv_pack_curr";
            title2 "%sysfunc(datetime(),nldatmap.)";
         run;

         options date;

      %end;

   %mend makeReport;

   proc datasets lib = work nolist nowarn memtype = (data view);
      delete valid_pack_inventory;
   quit;

   %let etls_Source = &SYSLAST;

   %let m_excp = 0;

   %put %str(NOTE: Cannot create target table as view because error records are moved.);

   /*---- Begin checks for Custom, Missing Values, and Invalid Values  ----*/
   data work.valid_pack_inventory
         (keep = GEO_ID PROD_ID START_DT END_DT CLOSE_INV_UNITS
                         CLOSE_INV_AMT_AT_RETAIL CLOSE_INV_AMT_AT_COST CHANGE_FLG
                         GEO_HIER_SK PROD_HIER_SK ETLS_EXCEPTION_COND)
        work.etls_Errors
         (keep = GEO_ID PROD_ID START_DT END_DT CLOSE_INV_UNITS
                         CLOSE_INV_AMT_AT_RETAIL CLOSE_INV_AMT_AT_COST CHANGE_FLG
                         GEO_HIER_SK PROD_HIER_SK ETLS_EXCEPTION_COND);

      set &etls_Source end=eof;

      i_move = 0;
      retain i_excp &m_excp;

      /*---- Custom Validation: Negative CLOSE_INV_UNITS  ----*/
      if (CLOSE_INV_UNITS lt 0) then
      do;
         ETLS_EXCEPTION_COND = put(cats(ETLS_EXCEPTION_COND,'0a'x,"*","%etl_msg(mid=5328,args=close_inv_units 0)"),$180.);
      end;  /* if (CLOSE_INV_UNITS lt 0) */
      else
      do;
      end;  /* else */

      /*---- Custom Validation: Negative CLOSE_INV_AMT_AT_RETAIL  ----*/
      if (CLOSE_INV_AMT_AT_RETAIL lt 0) then
      do;
         ETLS_EXCEPTION_COND = put(cats(ETLS_EXCEPTION_COND,'0a'x,"*","%etl_msg(mid=5328,args=close_inv_amt_at_retail 0)"),$180.);
      end;  /* if (CLOSE_INV_AMT_AT_RETAIL lt 0) */
      else
      do;
      end;  /* else */

      /*---- Custom Validation: Invalid CLOSE_INV_AMT_AT_COST  ----*/
      if (CLOSE_INV_AMT_AT_COST lt 0) then
      do;
         ETLS_EXCEPTION_COND = put(cats(ETLS_EXCEPTION_COND,'0a'x,"*","%etl_msg(mid=5328,args=close_inv_amt_at_cost 0)"),$180.);
      end;  /* if (CLOSE_INV_AMT_AT_COST lt 0) */
      else
      do;
      end;  /* else */

      /*---- Custom Validation: Invalid START_DT  ----*/
      if (START_DT gt "&g_curr_start_dt"d) then
      do;
         ETLS_EXCEPTION_COND = put(cats(ETLS_EXCEPTION_COND,'0a'x,"*","%etl_msg(mid=5322,args=start_dt &g_curr_start_dt)"),$180.);
      end;  /* if (START_DT gt "&g_curr_start_dt"d) */
      else
      do;
      end;  /* else */

      /*---- Custom Validation: Change_flg=Delete  ----*/
      if (upcase(CHANGE_FLG)=upcase("&etl_delete_flg")) then
      do;
         ETLS_EXCEPTION_COND = Put(cats(ETLS_EXCEPTION_COND,'0a'x,"*","%etl_msg(mid=5302,args=Change_flg=&etl_delete_flg)"),$180.);
      end;  /* if (upcase(CHANGE_FLG)=upcase("&etl_delete_flg")) */
      else
      do;
      end;  /* else */

      /*---- Custom Validation: START_DT > END_DT  ----*/
      if (START_DT > END_DT) then
      do;
         ETLS_EXCEPTION_COND = put(cats(ETLS_EXCEPTION_COND,'0a'x,"*","%etl_msg(mid=5322,args=START_DT END_DT)"),$180.);
      end;  /* if (START_DT > END_DT) */
      else
      do;
      end;  /* else */

      /*---- Custom Validation: All exception records  ----*/
      if (ETLS_EXCEPTION_COND ne "") then
      do;
         i_move = 1;
      end;  /* if (ETLS_EXCEPTION_COND ne "") */
      else
      do;
      end;  /* else */

      /* move record to error file  */
      if (i_move) then
         output work.etls_Errors;
      /* move record to output file  */
      else
         output work.valid_pack_inventory;

      if eof then
      do;
         /* create macro flags  */
         call symput("m_excp", put(i_excp, 1.));
      end;

   run;

   %rcSet(&syserr);

   %let etls_Source = work.valid_pack_inventory;

   /* add ETL_Error_JobRunTime variable to error table  */
   data work.etls_Errors;
      set work.etls_Errors;
      format ETL_Error_JobRunTime datetime16.;
      ETL_Error_JobRunTime = "&etls_stepStartTime"dt;
   run;

   /* update the errors table  */
   proc append base = DI_MON.STG_INVENTORY_PACK_X 
               data = work.ETLS_ERRORS_VW; 
   run; 
   proc append base = work._temp
               data = work.etls_Errors force;
   run;

   proc datasets lib = work nolist nowarn memtype = (data view);
      delete etls_Errors;
   quit;

   %goto STEP_CONTINUE;

   %STEP_ERR:

      %let sysrc = 9999;
      %rcSet(&sysrc);

   %STEP_CONTINUE:

%mend etls_Validate;
%etls_Validate;



/**  Step end Val_pack_inventory_fact **/

/*============================================================================*
 * Step:            Load_pack_inventory_fact                                  *
 * Transform:       Table Loader (Version 2.1)                                *
 * Description:                                                               *
 *                                                                            *
 * Source Table:    Data Validation Transform -                               *
 *                   work.valid_pack_inventory                                *
 * Target Table:    PACK_INVENTORY_FACT -                                     *
 *                   DI_DM.PACK_INVENTORY_FACT                                *
 *============================================================================*/

%let transformID = %quote(A5RCIHUB.BC0009QN);
%let trans_rc = 0;
%let etls_stepStartTime = %sysfunc(datetime(), datetime20.);

%let SYSLAST = %nrquote(work.valid_pack_inventory);

%let ETLS_SYSLAST = &SYSLAST;
/*---- Start of Pre-Process Code  ----*/

option obs=max;

%di_etl_keep_work_table(valid_pack_inventory);
/*---- End of Pre-Process Code  ----*/

%rcSet(&syserr);
%rcSet(&sqlrc);

%let SYSLAST = &ETLS_SYSLAST;

%global etls_tableExist;
%global etls_numIndex;
%global etls_lastTable;
%let etls_tableExist = -1;
%let etls_numIndex = -1;
%let etls_lastTable = &SYSLAST;

/*---- Define load data macro  ----*/

/* --------------------------------------------------------------
   Load Technique Selection: Update - Matching=ModifyUsingIndex, New=ModifyUsingIndex
   Blanks may replace non-blanks: true
   Constraint and index action selections: 'INIT','ASIS','ASIS','ASIS'
   Additional options selections...
      Set unmapped to missing on updates: false
   -------------------------------------------------------------- */
%macro etls_loader;

   %let etls_tableOptions = ;

   /* Get number of records in table  */
   %let etls_recCheckExist = 0;
   %let etls_recordsBefore = 0;
   %macro etls_recordCheck;
      %let etls_recCheckExist = %eval(%sysfunc(exist(DI_DM.PACK_INVENTORY_FACT, DATA)) or
            %sysfunc(exist(DI_DM.PACK_INVENTORY_FACT, VIEW)));

      %if (&etls_recCheckExist) %then
      %do;
         %local etls_syntaxcheck;
         %let etls_syntaxcheck = %sysfunc(getoption(syntaxcheck));
         /* Turn off syntaxcheck option to perform following steps  */
         options nosyntaxcheck;

         proc contents data = DI_DM.PACK_INVENTORY_FACT out = work.etls_contents(keep = nobs) noprint;
         run;

         data _null_;
            set work.etls_contents (obs = 1);
            call symput("etls_recordsBefore", left(put(nobs,32.)));
         run;

         proc datasets lib = work nolist nowarn memtype = (data view);
            delete etls_contents;
         quit;

         /* Reset syntaxcheck option to previous setting  */
         options &etls_syntaxcheck;
      %end;
   %mend etls_recordCheck;
   %etls_recordCheck;

   %let etls_table = %nrquote(PACK_INVENTORY_FACT);
   %let etls_lib   = %nrquote(DI_DM);

   /*---- Map the columns  ----*/
   proc datasets lib = work nolist nowarn memtype = (data view);
      delete W5MU39J9;
   quit;

   %put %str(NOTE: Mapping columns ...);
   proc sql;
      create view work.W5MU39J9 as
         select
            GEO_HIER_SK,
            PROD_HIER_SK,
            CLOSE_INV_AMT_AT_RETAIL as CLOSE_INV_VALUE_AT_RETAIL_AMT
               format = NLNUM12.2
               informat = NLNUM12.2,
            CLOSE_INV_AMT_AT_COST as CLOSE_INV_VALUE_AT_COST_AMT
               format = NLNUM12.2
               informat = NLNUM12.2,
            CLOSE_INV_UNITS
      from &etls_lastTable
      ;
   quit;

   %let SYSLAST = work.W5MU39J9;

   %let etls_lastTable = &SYSLAST;
   %let etls_tableOptions = ;

   /* Determine if the target table exists  */
   %let etls_tableExist = %eval(%sysfunc(exist(DI_DM.PACK_INVENTORY_FACT, DATA)) or
         %sysfunc(exist(DI_DM.PACK_INVENTORY_FACT, VIEW)));

   %let etls_hasUpdateIndex=0;

   /*---- Create a new table  ----*/
   %if (&etls_tableExist eq 0) %then
   %do;  /* if table does not exist  */

      %put %str(NOTE: Creating table ...);

      data DI_DM.PACK_INVENTORY_FACT;
         attrib GEO_HIER_SK length = 4;
         attrib PROD_HIER_SK length = 6;
         attrib CLOSE_INV_VALUE_AT_RETAIL_AMT length = 8
            format = NLNUM12.2
            informat = NLNUM12.2;
         attrib CLOSE_INV_VALUE_AT_COST_AMT length = 8
            format = NLNUM12.2
            informat = NLNUM12.2;
         attrib CLOSE_INV_UNITS length = 8
            format = NLNUM12.2
            informat = NLNUM12.2;
         call missing(of _all_);
         stop;
      run;

      %rcSet(&syserr);

      /*---- Create the integrity constraints for a table  ----*/
      %put %str(NOTE: Creating integrity constraints ...);
      proc datasets library=DI_DM nolist;
         modify PACK_INVENTORY_FACT;
            ic create not null (GEO_HIER_SK);
            ic create not null (PROD_HIER_SK);
            ic create PRIM_KEY = primary key (GEO_HIER_SK PROD_HIER_SK);
      quit;

      %rcSet(&syserr);

   %end;  /* if table does not exist  */

   data DI_DM.PACK_INVENTORY_FACT;

      set &etls_lastTable
             (rename=(CLOSE_INV_VALUE_AT_RETAIL_AMT = etls_temp0
                      CLOSE_INV_VALUE_AT_COST_AMT = etls_temp1
                      CLOSE_INV_UNITS = etls_temp2))
      ;

      modify DI_DM.PACK_INVENTORY_FACT
            key = PRIM_KEY;

      if _iorc_ in( %sysrc(_DSENMR),%sysrc(_DSENOM),%sysrc(_DSEMTR)) or
         _iorc_ eq %sysrc(_SOK) then
      do;
         CLOSE_INV_VALUE_AT_RETAIL_AMT = etls_temp0;
         CLOSE_INV_VALUE_AT_COST_AMT = etls_temp1;
         CLOSE_INV_UNITS = etls_temp2;

         /* if the record exists in the master, then replace it  */
         if _iorc_ eq %sysrc(_SOK) then
            replace;

         /* if the record does not exist in the master, then add it  */
         else
            output;
      end;

      _iorc_ = 0;
      _error_ = 0;

   run;

   %rcSet(&syserr);

   proc datasets lib = work nolist nowarn memtype = (data view);
      delete W5MU39J9;
   quit;

   /* Get number of records in table  */
   %let etls_recCheckExist = 0;
   %let etls_recordsAfter = 0;
   %macro etls_recordCheck;
      %let etls_recCheckExist = %eval(%sysfunc(exist(DI_DM.PACK_INVENTORY_FACT, DATA)) or
            %sysfunc(exist(DI_DM.PACK_INVENTORY_FACT, VIEW)));

      %if (&etls_recCheckExist) %then
      %do;
         %local etls_syntaxcheck;
         %let etls_syntaxcheck = %sysfunc(getoption(syntaxcheck));
         /* Turn off syntaxcheck option to perform following steps  */
         options nosyntaxcheck;

         proc contents data = DI_DM.PACK_INVENTORY_FACT out = work.etls_contents(keep = nobs) noprint;
         run;

         data _null_;
            set work.etls_contents (obs = 1);
            call symput("etls_recordsAfter", left(put(nobs,32.)));
         run;

         proc datasets lib = work nolist nowarn memtype = (data view);
            delete etls_contents;
         quit;

         /* Reset syntaxcheck option to previous setting  */
         options &etls_syntaxcheck;
      %end;
   %mend etls_recordCheck;
   %etls_recordCheck;

%mend etls_loader;
%etls_loader;

/**  Step end Load_pack_inventory_fact **/

%let etls_endTime = %sysfunc(datetime(),datetime.);

/*---- Job status macro  ----*/
%etls_sendJobStatus(source_table=DIDM_STG.STG_INVENTORY);
