/*
SUPPORT: sinprs Swain, Pradeep
*/
/******************************************************************************
 * Job:             Load_STG2DM_Size_translation            A5EEB9WX.BG00004E *
 * Description:                                                               *
 *                                                                            *
 * Metadata Server: DES10                                                     *
 * Port:            8561                                                      *
 * Location:        /Shared Data/DI/Jobs                                      *
 *                                                                            *
 * Server:          SASApp                                  A5EEB9WX.AQ000002 *
 *                                                                            *
 * Source Tables:   STG_SIZE_TRANSLATION -                  A5EEB9WX.B90001KH *
 *                   DIDM_STG.STG_SIZE_TRANSLATION                            *
 *                  PRODUCT_DM - DI_DM.PRODUCT_DM           A5EEB9WX.B90001KW *
 * Target Table:    SIZE_TRANSLATION -                      A5EEB9WX.B90001GC *
 *                   DI_DM2.SIZE_TRANSLATION                                  *
 *                                                                            *
 * Generated on:    Friday, May 8, 2009 11:29:18 AM EDT                       *
 * Generated by:    sinprs@CARYNT                                             *
 * Version:         SAS Data Integration Studio 4.2                           *
 ******************************************************************************/

/* General macro variables  */
%let jobID = %quote(A5EEB9WX.BG00004E);
%let etls_jobName = %nrquote(Load_STG2DM_Size_translation);
%let etls_userID = %nrquote(sinprs@CARYNT);

/* Setup to capture return codes  */
%global job_rc trans_rc sqlrc;
%let sysrc = 0;
%let job_rc = 0;
%let trans_rc = 0;
%let sqlrc = 0;
%global syserr;

%macro rcSet(error);
   %if (&error gt &trans_rc) %then
      %let trans_rc = &error;
   %if (&error gt &job_rc) %then
      %let job_rc = &error;
%mend rcSet;

%macro rcSetDS(error);
   if &error gt input(symget('trans_rc'),12.) then
      call symput('trans_rc',trim(left(put(&error,12.))));
   if &error gt input(symget('job_rc'),12.) then
      call symput('job_rc',trim(left(put(&error,12.))));
%mend rcSetDS;

/* Setup for capturing job status  */
%let etls_startTime = %sysfunc(datetime(),datetime.);
%let etls_recordsBefore = 0;
%let etls_recordsAfter = 0;
%let etls_lib = 0;
%let etls_table = 0;

%global etls_debug;
%macro etls_setDebug;
   %if %str(&etls_debug) ne 0 %then
      OPTIONS MPRINT%str(;);
%mend;
%etls_setDebug;

/*---- Start of Pre-Process Code  ----*/

/**/
%etl_initjobparam (monlib=di_mon, jobname=%nrquote(&etls_jobName));
/**/
%etl_initchangeflags;
/*---- End of Pre-Process Code  ----*/

%rcSet(&syserr);
%rcSet(&sqlrc);

/*============================================================================*
 * Step:            SAS Extract                             A5EEB9WX.BI0000CL *
 * Transform:       Extract                                                   *
 * Description:                                                               *
 *                                                                            *
 * Source Table:    PRODUCT_DM - DI_DM.PRODUCT_DM           A5EEB9WX.B90001KW *
 * Target Table:    Extract Target - work.EXT_PRODUCT_DM    A5EEB9WX.BK0000AS *
 *============================================================================*/

%let transformID = %quote(A5EEB9WX.BI0000CL);
%let trans_rc = 0;
%let etls_stepStartTime = %sysfunc(datetime(), datetime20.);

%let SYSLAST = %nrquote(DI_DM.PRODUCT_DM);

/*---- Map the columns  ----*/
proc datasets lib = work nolist nowarn memtype = (data view);
   delete EXT_PRODUCT_DM;
quit;

%put %str(NOTE: Mapping columns ...);
proc sql;
   create view work.EXT_PRODUCT_DM as
      select
         (max(PROD_HIER_SK)) as PROD_HIER_SK length = 6,
         PROD_ID,
         (max(STD_HIER_PROD_LVL)) as STD_HIER_PROD_LVL length = 4
   from &SYSLAST
   group by
      PROD_ID
   ;
quit;

%let SYSLAST = work.EXT_PRODUCT_DM;

%rcSet(&sqlrc);



/**  Step end SAS Extract **/

/*---- Start of User Written Code  ----*/

/* Get database options for bulkload */

%global g_trans_options;
%macro get_trans_options;
   %if &di_trans_vendor eq %str(DB2) %then %do;
      %let g_trans_options = %str(BULKLOAD=YES BL_METHOD=CLILOAD BL_ALLOW_READ_ACCESS=YES);
   %end;
   %else %if &di_trans_vendor eq %str(ORACLE) %then %do;
     %let bl_dir=%sysfunc(pathname(work))  ;
     %let bl_dir=&bl_dir./;
     %let g_trans_options = %str(BULKLOAD=YES BL_LOAD_METHOD=APPEND bl_default_dir="&bl_dir") ;
   %end;
%mend get_trans_options;
%get_trans_options;

%global G_SZ_TRANS_PROD_LVL;

%macro etl_check_exist;

   %let abort = 0 ;

   /* Check for existence of DI_DM.PRODUCT_DM dataset */
   %if NOT %sysfunc(exist(DI_DM.PRODUCT_DM)) %then
   %do;
      %let etl_msg = %etl_msg(mid=5005,args=DI_DM.PRODUCT_DM);
      %put &etl_msg;
      %let abort = 1 ;
   %end;

   /* Check for existence of DI_DM2.SRC_SIZE_CD dataset */
   %if NOT %sysfunc(exist(DI_DM2.SRC_SIZE_CD)) %then
   %do;
      %let etl_msg = %etl_msg(mid=5005,args=DI_DM2.SRC_SIZE_CD);
      %put &etl_msg;
      %let abort = 2 ;
   %end;

   /* Check for existence of DI_DM2.STD_SIZE_CD dataset */
   %if NOT %sysfunc(exist(DI_DM2.STD_SIZE_CD)) %then
   %do;
      %let etl_msg = %etl_msg(mid=5005,args=DI_DM2.STD_SIZE_CD);
      %put &etl_msg;
      %let abort = 3 ;
   %end;

   /*Get size translation prod level from global settings*/
   PROC SQL noprint;
      select spec_value into :G_SZ_TRANS_PROD_LVL
      from di_dm.global_setting
      where upcase(spec_nm)='SIZE_TRANSLATION_PROD_LVL';
   QUIT;

   %if &g_sz_trans_prod_lvl eq %str() %then
   %do;
      %let etl_msg = %etl_msg(mid=5007,args=SIZE_TRANSLATION_PROD_LVL GLOBAL_SETTING);
      %put &etl_msg;
      %let abort = 7 ;
   %end;

   %put G_SZ_TRANS_PROD_LVL=&g_sz_trans_prod_lvl;

   %if &abort gt 0 %then %do;
      %etl_abort;
   %end;

%mend etl_check_exist;

%etl_check_exist;

proc datasets lib = work nolist nowarn memtype = (data view);
   delete src_size_cd_vw _temp;
quit;

/* Create Lookup informat for src_size_cd */
data _temp;
run;

data work.src_size_cd_vw/view=work.src_size_cd_vw;
set DI_DM2.SRC_SIZE_CD _temp end=eof;
   SRC_SIZE_CD_TXT = left(put(SRC_SIZE_CD,32.));
   ID_VALUE = 1;
   if NOT missing(SRC_SIZE_CD) then output;
   if eof then
   do;
         SRC_SIZE_CD_TXT = "";
         ID_VALUE = -1;
         output;
   end;
run;

%etl_create_dim_lkp_informat
               (trantable= work.src_size_cd_vw
               ,trankey  = src_size_cd_txt
               ,tranvalue= id_value
               ,fmtname  = src_size_cd
               ,fmtlib   = work
               );

proc datasets lib = work nolist nowarn memtype = (data view);
   delete src_size_cd_vw std_size_cd_vw _temp;
quit;

/* Create Lookup informat for std_size_cd */
data _temp;
run;

data work.std_size_cd_vw/view=work.std_size_cd_vw;
set DI_DM2.STD_SIZE_CD _temp end=eof;
   STD_SIZE_CD_TXT = left(put(STD_SIZE_CD,32.));
   ID_VALUE = 1;
   if NOT missing(STD_SIZE_CD) then output;
   if eof then
   do;
         STD_SIZE_CD_TXT = "";
         ID_VALUE = -1;
         output;
   end;
run;

%etl_create_dim_lkp_informat
               (trantable= work.std_size_cd_vw
               ,trankey  = std_size_cd_txt
               ,tranvalue= id_value
               ,fmtname  = std_size_cd
               ,fmtlib   = work
               );

proc datasets lib = work nolist nowarn memtype = (data view);
   delete std_size_cd_vw _temp;
quit;

/*---- End of User Written Code  ----*/

/*============================================================================*
 * Step:            Lookup                                  A5EEB9WX.BI0000CN *
 * Transform:       Lookup                                                    *
 * Description:                                                               *
 *                                                                            *
 * Source Tables:   User Written Code Target -              A5EEB9WX.BK0000AT *
 *                   DIDM_STG.STG_SIZE_TRANSLATION                            *
 *                  Extract Target - work.EXT_PRODUCT_DM    A5EEB9WX.BK0000AS *
 * Target Table:    Lookup Target -                         A5EEB9WX.BK0000AU *
 *                   work.STG_SIZE_TRANSLATION_LKP                            *
 *============================================================================*/

%let transformID = %quote(A5EEB9WX.BI0000CN);
%let trans_rc = 0;
%let etls_stepStartTime = %sysfunc(datetime(), datetime20.);

%let runTime = %sysfunc(datetime());

proc datasets lib = work nolist nowarn memtype = (data view);
   delete STG_SIZE_TRANSLATION_LKP;
quit;

/*---- Begin DATA step to perform lookups  ----*/
DATA  work.STG_SIZE_TRANSLATION_LKP
         ( keep = SRC_SIZE_CD PROD_ID STD_SIZE_CD PROD_HIER_SK PROD_LVL )
      ;

   attrib SRC_SIZE_CD length=6
          PROD_ID length=$32
          STD_SIZE_CD length=6
          PROD_HIER_SK length=6
          PROD_LVL length=4
          STD_HIER_PROD_LVL length= 4
          ;

   retain missing0-missing0 0;

   etls_assign_target_value = 0;
   /* Build hash objects from lookup tables before reading first source row  */
   if (_n_ = 1) then
   do;
      /* Build hash h0 from lookup table work.EXT_PRODUCT_DM */
      nlobs = .;
      dsid = open("work.EXT_PRODUCT_DM");
      if (dsid > 0) then
      do;
         if ( attrc(dsid, 'MTYPE') = 'DATA' ) then
            nlobs = attrn(dsid, 'NLOBS');
         else
            nlobs = -1;

         dsid = close(dsid);
         if (nlobs ^= 0) then
         do;
            if (nlobs > 0) then
               exponent = ceil(log2(nlobs));
            else
               exponent = 8;
            declare hash h0(dataset: "work.EXT_PRODUCT_DM", hashexp: exponent);
            h0.defineKey( "PROD_ID");
            h0.defineData( "PROD_HIER_SK",
                           "STD_HIER_PROD_LVL");
            h0.defineDone();
            if (nlobs = -1) then
            do;
               if (h0.Num_Items < 1) then
               do;
                  put "NOTE: Lookup table is empty: work.EXT_PRODUCT_DM";
                  missing0 = 98;
               end;
            end;
         end;
         else
         do;
            put "NOTE: Lookup table is empty: work.EXT_PRODUCT_DM";
            missing0 = 98;
         end;
      end;
      else
      do;
         put "NOTE: Lookup table does not exist or cannot be opened:"
              " work.EXT_PRODUCT_DM";

         put "NOTE: Abort action indicated, condition= Lookup table missing:"
              " work.EXT_PRODUCT_DM";

         abort 3;
      end;

      call missing (PROD_ID, PROD_HIER_SK, STD_HIER_PROD_LVL);

   end; /* All hash objects have been defined */

   /* Read a row from the source table  */
   set DIDM_STG.STG_SIZE_TRANSLATION end = eof;

   /* Is the current key value stored in hash h0?  */
   if (missing0 < 1) then
   do;
      rc0 = h0.find();
      PROD_LVL = STD_HIER_PROD_LVL;
   end;
   else
      rc0 = missing0;

   /* Examine success of lookups  */
   if ( rc0=0 ) then
   do;
      /* Write row to target  */
      output work.STG_SIZE_TRANSLATION_LKP;
      return;
   end;
   else
   do;
      error_total + 1;

      if (rc0 ^= 0) then
      do;
         exception_total + 1;
         if (rc0 = 98) then
         do;
            /* Check: Lookup table has no rows-Write exception row  */
            /* Check: Lookup table has no rows-Set target columns to value  */
            /* Check: Lookup table has no rows-Set target columns to missing  */
            call missing (PROD_HIER_SK, PROD_LVL);
            etls_assign_target_value = 1;
         end;

         else
         do;
            /* Check: Lookup value not found-Set target columns to missing  */
            call missing (PROD_HIER_SK, PROD_LVL);
            etls_assign_target_value = 1;
         end;
      end;

      /* Set target columns to value/missing requested?  */
      if (etls_assign_target_value eq 1) then
         /* Write row to target  */
         output work.STG_SIZE_TRANSLATION_LKP;

   end; /* One or more lookups failed */

   if (eof = 1) then
   do;
      put "Source records with errors: " error_total ;
      put "Total lookup exceptions: " exception_total ;
   end;

run;

%rcSet(&syserr);



/**  Step end Lookup **/

/*============================================================================*
 * Step:            Chk_Duplicates                          A5EEB9WX.BI0000CO *
 * Transform:       User Written                                              *
 * Description:                                                               *
 *                                                                            *
 * Source Table:    Lookup Target -                         A5EEB9WX.BK0000AU *
 *                   work.STG_SIZE_TRANSLATION_LKP                            *
 * Target Table:    User Written Code Target -              A5EEB9WX.BK0000AV *
 *                   work.STG_SIZE_TRANSLATION_NODUP                          *
 *                                                                            *
 * User Written:    Check duplicates                                          *
 *============================================================================*/

%let transformID = %quote(A5EEB9WX.BI0000CO);
%let trans_rc = 0;
%let etls_stepStartTime = %sysfunc(datetime(), datetime20.);

%let SYSLAST = %nrquote(work.STG_SIZE_TRANSLATION_LKP);

%let ETLS_SYSLAST = &SYSLAST;
/*---- Start of Pre-Process Code  ----*/

%di_etl_keep_work_table(STG_SIZE_TRANSLATION_LKP);
/*---- End of Pre-Process Code  ----*/

%rcSet(&syserr);
%rcSet(&sqlrc);

%let SYSLAST = &ETLS_SYSLAST;

%let _INPUT_count = 1;
%let _INPUT = work.STG_SIZE_TRANSLATION_LKP;
%let _INPUT_connect = ;
%let _INPUT_engine = ;
%let _INPUT_memtype = DATA;
%let _INPUT_options = %nrquote();
%let _INPUT_alter = %nrquote();
%let _INPUT_path = /Lookup Target(WorkTable);
%let _INPUT_type = 1;

%let _INPUT1 = work.STG_SIZE_TRANSLATION_LKP;
%let _INPUT1_connect = ;
%let _INPUT1_engine = ;
%let _INPUT1_memtype = DATA;
%let _INPUT1_options = %nrquote();
%let _INPUT1_alter = %nrquote();
%let _INPUT1_path = /Lookup Target(WorkTable);
%let _INPUT1_type = 1;

%let _OUTPUT_count = 1;
%let _OUTPUT = work.STG_SIZE_TRANSLATION_NODUP;
%let _OUTPUT_connect = ;
%let _OUTPUT_engine = ;
%let _OUTPUT_memtype = DATA;
%let _OUTPUT_options = %nrquote();
%let _OUTPUT_alter = %nrquote();
%let _OUTPUT_path = /User Written Code Target(WorkTable);
%let _OUTPUT_type = 1;
/* List of target columns to keep  */
%let _OUTPUT_keep = SRC_SIZE_CD PROD_ID STD_SIZE_CD ETLS_EXCEPTION_COND PROD_HIER_SK
        PROD_LVL;


%let _OUTPUT1 = work.STG_SIZE_TRANSLATION_NODUP;
%let _OUTPUT1_connect = ;
%let _OUTPUT1_engine = ;
%let _OUTPUT1_memtype = DATA;
%let _OUTPUT1_options = %nrquote();
%let _OUTPUT1_alter = %nrquote();
%let _OUTPUT1_path = /User Written Code Target(WorkTable);
%let _OUTPUT1_type = 1;
/* List of target columns to keep  */
%let _OUTPUT1_keep = SRC_SIZE_CD PROD_ID STD_SIZE_CD ETLS_EXCEPTION_COND PROD_HIER_SK
        PROD_LVL;


/*---- Start of User Written Code  ----*/

/* SORT for removing duplicates */

proc datasets lib = work nolist nowarn memtype = (data view);
   delete sort_tab;
quit;

proc sort data=&SYSLAST out=work.sort_tab;
   by SRC_SIZE_CD PROD_HIER_SK;
run;

data &_OUTPUT;
   set work.sort_tab;
   attrib ETLS_EXCEPTION_COND length=$255;
   by SRC_SIZE_CD PROD_HIER_SK;
   if not(last.PROD_HIER_SK) then do;
      ETLS_EXCEPTION_COND = put(cats(ETLS_EXCEPTION_COND,'0a'x," * %etl_msg(mid=5351)"),$255.);
   end;
run;

proc datasets lib = work nolist nowarn memtype = (data view);
   delete sort_tab;
quit;
/*---- End of User Written Code  ----*/

%rcSet(&syserr);
%rcSet(&sqlrc);


/**  Step end Chk_Duplicates **/

/*============================================================================*
 * Step:            Data Validation                         A5EEB9WX.BI0000CP *
 * Transform:       Data Validation                                           *
 * Description:                                                               *
 *                                                                            *
 * Source Table:    User Written Code Target -              A5EEB9WX.BK0000AV *
 *                   work.STG_SIZE_TRANSLATION_NODUP                          *
 * Target Table:    Data Validation Target -                A5EEB9WX.BK0000AW *
 *                   work.STG_SIZE_TRANSLATION_VALID                          *
 *============================================================================*/

%let transformID = %quote(A5EEB9WX.BI0000CP);
%let trans_rc = 0;
%let etls_stepStartTime = %sysfunc(datetime(), datetime20.);

%let SYSLAST = %nrquote(work.STG_SIZE_TRANSLATION_NODUP);

%let ETLS_SYSLAST = &SYSLAST;
/*---- Start of Pre-Process Code  ----*/

%di_etl_keep_work_table(STG_SIZE_TRANSLATION_NODUP);
proc datasets lib = work nolist nowarn memtype = (data view);
   delete stg_size_translation_x;
quit;
/*---- End of Pre-Process Code  ----*/

%rcSet(&syserr);
%rcSet(&sqlrc);

%let SYSLAST = &ETLS_SYSLAST;

%macro etls_Validate;

   %let runTime = %sysfunc(datetime());

   /* Create the Exception Report  */
   %macro makeReport;

      %if (&m_excp eq 1) %then
      %do;
         options nodate;

         proc sort data = work.etls_Exceptions;
            by x_row;
         run;

         proc print data = work.etls_Exceptions noobs label;
            title1 "Exceptions Found in Table: work.STG_SIZE_TRANSLATION_NODUP";
            title2 "%sysfunc(datetime(),nldatmap.)";
         run;

         options date;

      %end;

   %mend makeReport;

   proc datasets lib = work nolist nowarn memtype = (data view);
      delete STG_SIZE_TRANSLATION_VALID;
   quit;

   proc datasets lib = work nolist nowarn memtype = (data view);
      delete etls_STG_SIZE_TRANSLATION_VALID;
   quit;

   %let etls_Source = &SYSLAST;

   %let m_excp = 0;

   /*---- Begin checks for Custom, Missing Values, and Invalid Values  ----*/
   data work.etls_STG_SIZE_TRANSLATION_VALID
         (keep = SRC_SIZE_CD PROD_ID STD_SIZE_CD ETLS_EXCEPTION_COND PROD_HIER_SK
                         PROD_LVL)
        work.etls_Errors
         (keep = SRC_SIZE_CD PROD_ID STD_SIZE_CD ETLS_EXCEPTION_COND PROD_HIER_SK
                         PROD_LVL)
        work.etls_Exceptions
         (keep = x_row x_column x_type x_action x_note);

      /* Define lengths for columns with longer lengths in target  */
      attrib PROD_HIER_SK length = 8;

      set &etls_Source end=eof;

      attrib x_row    length = 4    label = "Row Number";
      attrib x_column length = $100 label = "Column Name";
      attrib x_type   length = $50  label = "Type of Exception";
      attrib x_action length = $50  label = "Action Taken";
      attrib x_note   length = $200 label = "Notes";

      retain x_note "";
      x_row = _n_;

      i_move = 0;
      retain i_excp &m_excp;

      /*---- Custom Validation: Prod_ID Lkp Check  ----*/
      if (PROD_HIER_SK IN( -1, 0, .)) then
      do;
         ETLS_EXCEPTION_COND = put(cats(ETLS_EXCEPTION_COND,'0a'x,"%etl_msg(mid=5306,args=PROD_ID)"),$255.);
         i_move = 1;
      end;  /* if (PROD_HIER_SK IN( -1, 0, .)) */
      else
      do;
      end;  /* else */

      /*---- Custom Validation: PROD_LVL should be at GLOBAL_SETTINGS  ----*/
      /*----  size_trans_prod_lvl                                      ----*/
      if (PROD_LVL ne input(symget('g_sz_trans_prod_lvl'),32.)) then
      do;
         ETLS_EXCEPTION_COND = put(cats(ETLS_EXCEPTION_COND,'0a'x,"%etl_msg(mid=5315,args=PROD_LVL-of-PROD_ID SIZE_TRANS_PROD_LVL )"),$255.);
         i_move = 1;
      end;  /* if (PROD_LVL ne input(symget('g_sz_trans_prod_lvl'),32.)) */
      else
      do;
      end;  /* else */

      /*---- Custom Validation: Invalid values check  ----*/
      if (SRC_SIZE_CD  < 0  | STD_SIZE_CD < 0) then
      do;
         ETLS_EXCEPTION_COND = put(cats(ETLS_EXCEPTION_COND,'0a'x,"%etl_msg(mid=5305,args=SRC_SIZE_CD/STD_SIZE_CD)"),$255.);
         i_move = 1;
      end;  /* if (SRC_SIZE_CD  < 0  | STD_SIZE_CD < 0) */
      else
      do;
      end;  /* else */

      /*---- Custom Validation: Invalid SRC_SIZE_CD  ----*/
      if (INPUT(left(put(SRC_SIZE_CD,32.)) ,SRC_SIZE_CD.) eq -1) then
      do;
         ETLS_EXCEPTION_COND = put(cats(ETLS_EXCEPTION_COND,'0a'x,"%etl_msg(mid=5306,args=SRC_SIZE_CD)"),$255.);
         i_move = 1;
      end;  /* if (INPUT(left(put(SRC_SIZE_CD,32.)) ,SRC_SIZE_CD.) eq -1) */
      else
      do;
      end;  /* else */

      /*---- Custom Validation: Invalid STD_SIZE_CD  ----*/
      if (INPUT(left(put(STD_SIZE_CD,32.)) ,STD_SIZE_CD.) eq -1) then
      do;
         ETLS_EXCEPTION_COND = put(cats(ETLS_EXCEPTION_COND,'0a'x,"%etl_msg(mid=5306,args=STD_SIZE_CD)"),$255.);
         i_move = 1;
      end;  /* if (INPUT(left(put(STD_SIZE_CD,32.)) ,STD_SIZE_CD.) eq -1) */
      else
      do;
      end;  /* else */

      /*---- Custom Validation: Remove Duplicate  ----*/
      if (ETLS_EXCEPTION_COND ne "") then
      do;
         i_move = 1;
      end;  /* if (ETLS_EXCEPTION_COND ne "") */
      else
      do;
      end;  /* else */

      /* move record to error file  */
      if (i_move) then
         output work.etls_Errors;
      /* move record to output file  */
      else
         output work.etls_STG_SIZE_TRANSLATION_VALID;

      if eof then
      do;
         /* create macro flags  */
         call symput("m_excp", put(i_excp, 1.));
      end;

   run;

   %rcSet(&syserr);

   %let etls_Source = work.etls_STG_SIZE_TRANSLATION_VALID;

   /* add ETL_Error_JobRunTime variable to error table  */
   data work.etls_Errors;
      set work.etls_Errors;
      format ETL_Error_JobRunTime datetime16.;
      ETL_Error_JobRunTime = "&etls_stepStartTime"dt;
   run;

   /* update the errors table  */
   proc append base = WORK.STG_SIZE_TRANSLATION_X
               data = work.etls_Errors force;
   run;

   proc datasets lib = work nolist nowarn memtype = (data view);
      delete etls_Errors;
   quit;

   %makeReport;

   /*---- Map the columns  ----*/
   proc datasets lib = work nolist nowarn memtype = (data view);
      delete STG_SIZE_TRANSLATION_VALID;
   quit;

   %put %str(NOTE: Mapping columns ...);
   proc sql;
      create table work.STG_SIZE_TRANSLATION_VALID as
         select
            SRC_SIZE_CD,
            PROD_ID,
            STD_SIZE_CD,
            ETLS_EXCEPTION_COND,
            PROD_HIER_SK length = 8,
            PROD_LVL
      from work.etls_STG_SIZE_TRANSLATION_VALID
      ;
   quit;

   %let SYSLAST = work.STG_SIZE_TRANSLATION_VALID;

   %goto STEP_CONTINUE;

   %STEP_ERR:

      %let sysrc = 9999;
      %rcSet(&sysrc);

   %STEP_CONTINUE:

   proc datasets lib = work nolist nowarn memtype = (data view);
      delete etls_STG_SIZE_TRANSLATION_VALID;
   quit;

%mend etls_Validate;
%etls_Validate;

/*---- Start of Post-Process Code  ----*/

proc append base=DI_MON.STG_SIZE_TRANSLATION_X
            data=work.STG_SIZE_TRANSLATION_X(drop=PROD_HIER_SK);
run;
/*---- End of Post-Process Code  ----*/

%rcSet(&syserr);
%rcSet(&sqlrc);



/**  Step end Data Validation **/

/*---- Start of User Written Code  ----*/

%di_etl_keep_work_table(STG_SIZE_TRANSLATION_VALID);

%global etls_tableExist;
%let etls_tableExist = -1;

%let etls_recordsBefore = 0;
%let etls_table = %nrquote(SIZE_TRANSLATION);
%let etls_lib   = %nrquote(DI_DM2);

%macro etls_recordCheck(varname);
   %let etls_recCheckExist = %eval(%sysfunc(exist(DI_DM2.SIZE_TRANSLATION, DATA)) or
      %sysfunc(exist(DI_DM2.SIZE_TRANSLATION, VIEW)));

   %if (&etls_recCheckExist) %then %do;
      proc sql noprint;
         select count(*) into :&varname from DI_DM2.SIZE_TRANSLATION;
      quit;
   %end;
%mend etls_recordCheck;

/*---- Define load data macro  ----*/
%macro etls_loader;

   %local etls_match_rec_count vars_to_keep;
   %let etls_match_rec_count = 0;

   /* Determine if the target table exists  */
   %let etls_tableExist = %eval(%sysfunc(exist(DI_DM2.SIZE_TRANSLATION, DATA)) or
         %sysfunc(exist(DI_DM2.SIZE_TRANSLATION, VIEW)));

   %let etls_hasUpdateIndex=0;

   %if (&etls_tableExist eq 0) %then
   %do;  /* if table does not exist  */
      %etl_msg(mid=5005,args=DI_DM2.SIZE_TRANSLATION);
      %etl_abort;
   %end;  /* if table does not exist  */

   /* Get number of records in table  */
   %etls_recordCheck(etls_recordsBefore);

   proc datasets lib = work nolist nowarn memtype = (data view);
      delete etls_newrecords etls_existrecords size_translation;
   quit;

   %if (&etls_recordsBefore gt 0) %then %do;  /* if has rows */
      %di_dm_trans_extract(
         di_trans      = di_dm2,
         trans_table   = size_translation,
         all_trans     = 0,
         _output_table = size_translation);

      proc sort data=size_translation;
        by src_size_cd prod_hier_sk;
     run;

     %let vars_to_keep = %di_util_column_names(size_translation);

      /* create a temp table of matching records & new records from source and transactio table */
     data etls_existrecords etls_newrecords(drop=size_trans_sk);
        merge size_translation(in=dm) stg_size_translation_valid(in=stg);
       by src_size_cd prod_hier_sk;
       keep &vars_to_keep;
       if dm and stg then output etls_existrecords;
         if not dm then output etls_newrecords;
      run;

      %rcSet(&syserr);
      %let etls_match_rec_count = %di_util_nobs(etls_existrecords);
   %end; /* if has rows */
   %else
   %do; /* It has no rows */
      /* Create view on input dataset to append */
      proc sql;
         create table etls_newrecords as
         select src_size_cd length = 8,
                prod_hier_sk length = 8,
                std_size_cd length = 8
           from STG_SIZE_TRANSLATION_VALID;
      quit;
      %rcSet(&sqlrc);
   %end;

   /*--- Update existing data in the master table ----*/
   %if %sysfunc(exist(etls_existrecords, DATA)) %then %do;
   %if &etls_match_rec_count gt 0 %then
   %do;

      %di_dm_trans_delete(
         di_trans=di_dm2,
         trans_table=size_translation,
         delete_join_table=work.etls_existrecords,
         delete_join_cond=%str(a.size_trans_sk=b.size_trans_sk),
         chunk = 1);

      proc append base=di_dm2.size_translation(&g_trans_options) data=work.etls_existrecords;
      run;

      %rcSet(&syserr);
     %end;
   %end;
   /*---- Append new data  ----*/
   %di_dm_trans_append(
      di_trans=di_dm2,
      trans_table=size_translation,
      append_table=etls_newrecords);

   %rcSet(&syserr);

   /* Get number of records in table  */
   %let etls_recordsAfter = 0;
   %etls_recordCheck(etls_recordsAfter);

%mend etls_loader;
%etls_loader;

/*---- End of User Written Code  ----*/

%let etls_endTime = %sysfunc(datetime(),datetime.);

/*---- Job status macro  ----*/
%etls_sendjobstatus(source_table=didm_stg.stg_size_translation);

