/* 
SUPPORT: fumeri Meric, Fusun
*/ 
/****************************************************************************************
                               DEMAND INTELLIGENCE

NAME:          di_dm_prf_plan_extract

DESCRIPTION:   prf_plan table extract macro

APPLICATIONS:  SIZE PROFILE 

MODULE:        DM

INPUTS:
   data_lib                  - data_lib to the trans table
       
   prf_plan_sk               - profiling_project_sk

   prf_plan_status_cd        - statuses to be extracted
   
   _out                      - output table name
   
   _rc                       - process status
                  

HISTORY:       26OCT06   alchie   Started creation of macro
               15APR08   fumeri   2.2 Schema Changes
               18MAY09   fumeri   S0559745: RC changes 

****************************************************************************************/
%macro di_dm_prf_plan_extract(
   data_lib=,
   prf_scenario_sk =,
   prf_plan_sk =,
   prf_plan_status_cd =,
   filter_table=,
   _out =,
   debug=0,
   _rc =
   );

%local rc_plan this_macro;
%let this_macro=&SYSMACRONAME;
%let rc_plan=0;
%if &debug %then %do;
   %local start_time end_time;
   %di_util_get_time(_time = start_time);
%end;

*------check existance of the input data and arg------*;

%if %di_util_check_sym(sym=data_lib) > 0 %then %do;
    %di_util_put(mid=20004, args=DATA_LIB &this_macro.);
    %goto ERROREXIT;
%end;
%if %di_util_check_sym(sym=_out) > 0 %then %do;
    %di_util_put(mid=20004, args=_OUT &this_macro.);
    %goto ERROREXIT;
%end;

*------define filter conditions------*;
%local use_filter_table; %let use_filter_table=0;
%local filter_cond; %let filter_cond = %str(1 = 1);
%if "&prf_plan_sk" ne "" %then %do; 
   %let num_of_plans = %sysfunc(countw("&prf_plan_sk"));
   data work.prf_plan_filterz(drop=i);
       attrib plan_sk length=8;
       do i = 1 to &num_of_plans;
           plan_sk = input(scan("&prf_plan_sk",i),best12.);
           output;
       end;
   run;

   %let filter_cond = &filter_cond %str( and a.prf_plan_sk = b.plan_sk);                                                                  
   %let use_filter_table = 1; 
   %let filter_table=work.prf_plan_filterz;
%end;

%else %if "&prf_scenario_sk" ne "" %then %do;
   
   %let num_of_scenario = %sysfunc(countw("&prf_scenario_sk"));
   data work.prf_scenario_filterz(drop=i);
       attrib prf_scenario_sk length=8;
       do i = 1 to &num_of_scenario;
          prf_scenario_sk = input(scan("&prf_scenario_sk",i),best12.);
          output;
       end;
   run;

   %let filter_cond = &filter_cond %str( and a.prf_scenario_sk = b.prf_scenario_sk);                                                                                
   %let use_filter_table = 1; 
   %let filter_table=work.prf_scenario_filterz;
   
%end;

%else %if "&filter_table" ne "" %then %do;
   %let filter_cond = &filter_cond %str( and a.prf_scenario_sk = b.prf_scenario_sk);                                                                  
   %let use_filter_table = 1;  
%end;                                                            

%if "&prf_plan_status_cd" ne "" %then
   %let filter_cond = &filter_cond %str( and a.status_cd in (&prf_plan_status_cd));

*------trans table extraction------*;
%di_dm_trans_extract(
   di_trans= &data_lib,
   trans_table = prf_plan,
   _output_table = &_out,
   select_list = %str(a.*),
   %if &use_filter_table = 1 %then %do;
      all_trans=0,
      filter_table=&filter_table,
	%end;								   
   filter_cond = %str(&filter_cond));
   
%goto EXIT;

%ERROREXIT:
   %let rc_plan=2;


%EXIT:

   %local msg;
   %let msg=%str();

   %if &rc_plan > &DI_RC_WARNING or &syscc > 4 %then %do;
      %let rc_plan=&DI_RC_ERROR;
   %end;
   %if "&_rc" ne "" %then %do;
      %if &rc_plan > &&&_rc %then %let &_rc=&rc_plan;
      %let msg=%str(and a parent return code=&&&_rc.);       
   %end;

   %if &debug = 0 %then %do;
      *clean temp folder;
   %end;
   %else %do;
      %di_util_get_time(_time = end_time);
      %put NOTE: &this_macro. started at &start_time., ended at &end_time. with a local return code=&rc_plan. &msg.;
   %end;


%mend;
