

/****************************************************************************************
                               DEMAND INTELLIGENCE

NAME:          di_dm_trans_aster_loader

DESCRIPTION:   aster_loader is for bulkload of data to aster nCluster database

APPLICATIONS:  All

MODULE:        DM

INPUTS:        di_trans       - Trans Database Library
               trans_table    - Target table name
               load_table     - Table to load
               loader_exe     - loader to Use. aster_loader will be used by default.
               loader_options - Options for aster loader
               
               
               
OUTPUTS:       

GLOBAL VAR:    NA

USAGE:         

THREADED:      NA

DOCUMENT:      NA

COMMENTS:      
              

AUTHOR:        vekada  Kadari, Venu

SUPPORT:       vekada Kadari, Venu

HISTORY:       15Jan2010 vekada   Started creating the macro
****************************************************************************************/

%macro di_dm_trans_aster_loader(
   trans_table=,
   load_table=,
   loader_exe=,
   loader_options=
   )
;


   * supports only fully qualified writes;
   %di_util_assert_sym(sym=di_trans);
   %di_util_assert_sym(sym=trans_table);

   %local fname append_list;
   %di_dm_get_append_columns(ds=&load_table, separator=%str(,), _columns=append_list);

   %if &loader_exe= %then %do;
      %let loader_exe=nCluster_loader;
   %end;

   /* di_dm_get_aster_loader_columns(ds=&trans_table, separator=%str(,), _columns=aster_loader_list); */
   
   %local db_conn table_ddl date_columns;
   
   %let db_conn = %str(user=&DI_TRANS_USER password="&DI_TRANS_PASSWORD" database=&DI_TRANS_DATABASE server="&DI_TRANS_SERVER" );
   libname temp sasioast &db_conn connection=global;
  
   %let fname = %sysfunc(translate(%sysfunc(pathname(work))/aster_xyz.txt,/,\));

   /* Proc export will not work because of the header column */
   /* proc export data=&load_table outfile= "&fname" dbms=TAB replace; quit; */

   %ds2csv (data=&load_table, runmode=b, csvfile="&fname", colhead=n);

   /* Build String to Execute */
   %let exeString = &DI_BL_PATH.&loader_exe &loader_options -h &DI_TRANS_SERVER -d &DI_TRANS_DATABASE -U &DI_TRANS_USER -w &DI_TRANS_PASSWORD --csv -C &append_list &TRANS_TABLE "&fname" ;
   X &exeString;
;
%mend ;

%macro di_dm_get_append_columns(ds=, separator=, _columns=);

   %local lib table;

   %let lib = %scan(&ds, 1, %str(.));
   %let table = %scan(&ds, 2, %str(.));
   %if ("&table" = "") %then %do;
      %let table = &lib;
      %let lib = WORK;
   %end;

   proc sql;
      create table tmp_columns as 
      select upcase(name) as name
      from dictionary.columns 
      where libname = upcase("&lib") and memname = upcase("&table");
   quit;
 
   %local columns;
   %let columns=;
   data _null_;
      set tmp_columns;
      if (_n_ > 1) then call symput("columns", cat(trim(symget("columns")), "&separator", name));
      else call symput("columns", name);
   run;

   %let &_columns = &columns;

%mend di_dm_get_append_columns;

/*

* For tab separated data use '09'x ;
%ds2csv (data=work.temp, runmode=b, sepchar='09'x, csvfile=c:\aster\sample3_in.txt, colhead=n);


libname tests 'C:\testsdata';
data temp;
set tests.app_group;
drop app_group_sk;
run;

options mprint mlogic symbolgen;
%di_dm_trans_aster_loader(
   trans_table=app_group,
   load_table=work.temp,
   loader_exe=,
   loader_options=
   )
*/
