/* 
SUPPORT: sinprs Swain, Pradeep
*/ 
/****************************************************************************************
                                    DEMAND INTELLIGENCE
NAME        : di_etl_geo_prod_exception
DESCRIPTION : This macro is called by ETL job of Load_DM_Geo_prod_exception.
              This macro Finds Exceptions for Active GEO_PROD for PRICE_FACT_CURRENT, COST_FACT_CURRENT 
              and INVENTORY_FACT.
              PROD_DISCONTINUED_FLG = 1 if product is discontined in past and geo-prod is still active
                                    = 0 if product is not discontinued and geo_prod is active

              GEO_CLOSE_FLG = 1 if Geo is closed in past and geo_prod is still active
                            = 0 if Geo is open and geo_prod is active
              
INPUTS      : N.A
OUTPUTS     : N.A
GLOBAL VAR  : PROD_PARTITION_LVL
USAGE       : %di_etl_geo_prod_exception(di_dm_lib_path=\\puneasd10\MI32\Foundation\DI_DM\tests,
                                         part_sk=2,
                                         work_lib_path=D:\SAS_WORK\_TD4912, 
                                         current_date_sk=20601,
                                         active_prod_status=%str(1,3,5,6,7,8),
                                         use_landed_cost=1);
AUTHOR      : sinsoj
SUPPORT     : sinsoj Jain, Sourabh

HISTORY     : 28July2010 created
              26Aug2011 sinprs Added Prod_discontinued_flg and Geo_close_flg to Geo_prod_exception.
                               Changed Current_date_Sk to Prev_current_date_sk.
                               Added src_ref_dt as fiscal_end_dt of Prev_current_date_sk.
              10May2012 sinass Modified exception reporting on 'use_landed_cost'.
                               If use_landed_cost=1 and cost_fact do not contain a record for
                               COST_TYPE=1 or COST_TYPE=2 in cost_fact then report exception.                              
                               If GEO_PROD_EXCEPTION contains CURRENT_COST_FLG=0 and COST_TYPE=2
                               that mean COST_FACT do not contain landed cost for that Geo-prod.
****************************************************************************************/

%macro di_etl_geo_prod_exception(di_dm_lib_path=,
                                 part_sk=,
                                 work_lib_path=,
                                 prev_current_date_sk =,
                                 src_ref_dt = ,
                                 active_prod_status=,
                                 use_landed_cost=);

   data _null_;
      pid  = &SYSJOBID;
      put '************************************************************************';
      put '*   Attention: The Process ID (PID) of this SAS Session is: ' pid ;
      put '************************************************************************';
      put '_';
   run;

   %let syscc=0;

   %local temp_path partition_path etl_msg ds_obs;

   %etl_initjobparam (monlib=di_mon, jobname=%nrquote(Load_DM_Geo_prod_exception));

   libname GPE_DM "&di_dm_lib_path";
   libname GPE_WORK "&work_lib_path";

   %if %sysfunc(libref(GPE_DM)) ne 0 %then %do;
      %goto no_process_exit;
   %end;

   %if %sysfunc(libref(GPE_WORK)) ne 0 %then %do;
      %goto no_process_exit;
   %end;

   %let temp_path = %di_util_create_folder(relative_dir=1,
                                           parent_dir=&di_dm_lib_path);
   %let partition_path = %di_util_create_folder(relative_dir=&part_sk,
                                           parent_dir=&temp_path);


   libname GPE_PART "&partition_path";

   %di_util_lock(GPE_DM.GEO_PROD_EXCEPTION);
   data GPE_WORK.GEO_PROD_EXCEPTION_&part_sk;
      set GPE_DM.GEO_PROD_EXCEPTION;
      stop;
   run;
   %di_util_unlock(GPE_DM.GEO_PROD_EXCEPTION);


   /* Check for existence of GEO_PROD dataset */
   %if NOT %sysfunc(exist(GPE_PART.GEO_PROD)) %then %do;
      %let etl_msg = %etl_msg(mid=5005,args=GEO_PROD);
      %put &etl_msg;
      %goto no_process_exit;
   %end;
   %else %do;
      %let ds_obs = %di_util_nobs(GPE_PART.GEO_PROD);
      %if &ds_obs = 0 %then %do;
         %let etl_msg = %etl_msg(mid=5008,args=GEO_PROD);
         %put &etl_msg;
         %goto no_process_exit;
      %end;
   %end;

   /* Check for existence of PRICE_FACT_CURRENT dataset */
   %if NOT %sysfunc(exist(GPE_PART.PRICE_FACT_CURRENT)) %then %do;
      %let etl_msg = %etl_msg(mid=5203,args=PRICE_FACT_CURRENT);
      %put &etl_msg;
   %end;
   %else %do;
      %let ds_obs = %di_util_nobs(GPE_PART.PRICE_FACT_CURRENT);
      %if &ds_obs = 0 %then %do;
         %let etl_msg = %etl_msg(mid=5203,args=PRICE_FACT_CURRENT);
         %put &etl_msg;
      %end;
   %end;

   /* Check for existence of COST_FACT_CURRENT dataset */
   %if NOT %sysfunc(exist(GPE_PART.COST_FACT_CURRENT)) %then %do;
      %let etl_msg = %etl_msg(mid=5203,args=COST_FACT_CURRENT);
      %put &etl_msg;
   %end;
   %else %do;
      %let ds_obs = %di_util_nobs(GPE_PART.COST_FACT_CURRENT);
      %if &ds_obs = 0 %then %do;
         %let etl_msg = %etl_msg(mid=5203,args=COST_FACT_CURRENT);
         %put &etl_msg;
       %end;
   %end;

   /* Check for existence of INVENTORY_FACT dataset */
   %if NOT %sysfunc(exist(GPE_PART.INVENTORY_FACT)) %then %do;
      %let etl_msg = %etl_msg(mid=5203,args=INVENTORY_FACT);
      %put &etl_msg;
   %end;
   %else %do;
      %let ds_obs = %di_util_nobs(GPE_PART.INVENTORY_FACT);
      %if &ds_obs = 0 %then %do;
         %let etl_msg = %etl_msg(mid=5203,args=INVENTORY_FACT);
         %put &etl_msg;
       %end;
   %end;

   /* Code block to find exceptions */
   data GPE_WORK.GP_EXCEPTION_&part_sk._tmp
      (keep=GEO_HIER_SK PROD_HIER_SK GEO_PROD_STATUS INVENTORY_FLG CURRENT_PRICE_FLG 
            CURRENT_COST_FLG COST_TYPE PROD_DISCONTINUED_FLG);

      attrib INVENTORY_FLG length=$1 CURRENT_PRICE_FLG length=$1 CURRENT_COST_FLG length=$1 
             PROD_DISCONTINUED_FLG length =$1;
      
      retain PROD_DISCONTINUED_FLG '';
      
	       %if %sysfunc(exist(GPE_PART.inventory_fact)) or  %sysfunc(exist(GPE_PART.price_fact_current))
		       or %sysfunc(exist(GPE_PART.cost_fact_current)) %then %do;
			     merge 
		   %end;
		   %else %do;
		         set
		   %end;
            GPE_PART.geo_prod(where=(GEO_PROD_STATUS in (&active_prod_status))
                              in=ingeo)
			%if %sysfunc(exist(GPE_PART.inventory_fact)) %then %do;
            GPE_PART.inventory_fact(where =(DATE_SK = &prev_current_date_sk) in=ininf)
		    %end;
			%if %sysfunc(exist(GPE_PART.price_fact_current)) %then %do;
            GPE_PART.price_fact_current(in=inpr)
			%end;
            %if %sysfunc(exist(GPE_PART.cost_fact_current)) %then %do;
            GPE_PART.cost_fact_current(in=incf) 
            %end;
      ;
      by PROD_HIER_SK GEO_HIER_SK ;
      
      %if %sysfunc(exist(GPE_PART.inventory_fact)) %then %do;
      INVENTORY_FLG = cats(ininf);
	  %end;
	  %else %do;
	      INVENTORY_FLG=0;
	  %end;
      %if %sysfunc(exist(GPE_PART.price_fact_current)) %then %do;
      CURRENT_PRICE_FLG = cats(inpr);
	  %end;
	  %else %do;
	     CURRENT_PRICE_FLG=0;
	  %end;
      %if %sysfunc(exist(GPE_PART.cost_fact_current)) %then %do;
      CURRENT_COST_FLG = cats(incf);
	  %end;
	  %else %do;
	      CURRENT_COST_FLG = 0;
	  %end;
      if CURRENT_COST_FLG=0 then COST_TYPE=1;
      if first.prod_hier_sk then 
      do;
         PROD_DISCONTINUED_FLG = '0';
      
         set GPE_PART.product_dm(keep=prod_hier_sk prod_discontinued_dt) key=prod_hier_sk/unique;
         if _iorc_ eq 0 then 
         do;
            if prod_discontinued_dt ne . and prod_discontinued_dt < "&src_ref_dt"d 
               then 
                  PROD_DISCONTINUED_FLG = '1';
               else    
                  PROD_DISCONTINUED_FLG = '0';
         end;
         _iorc_=0;_error_=0;
      end;
      drop prod_discontinued_dt;
      if ingeo;
   run;
   
   data GPE_WORK.GP_EXCEPTION_&part_sk._tmp (drop=next_cost_type);
      set GPE_WORK.GP_EXCEPTION_&part_sk._tmp;
      by PROD_HIER_SK GEO_HIER_SK ;
      next_cost_type=.;
      %if &use_landed_cost=1 %then %do;
         if first.GEO_HIER_SK and last.GEO_HIER_SK and COST_TYPE=1 then 
         do;
            output;
            CURRENT_COST_FLG = '0';
            COST_TYPE=2;
            output;
         end;
         else if first.GEO_HIER_SK and last.GEO_HIER_SK and COST_TYPE=2 then 
         do;
            output;
            CURRENT_COST_FLG = '0';
            COST_TYPE=1;
            output;
         end;
         else if first.GEO_HIER_SK and last.GEO_HIER_SK and COST_TYPE=3 then 
         do;
            output;
            CURRENT_COST_FLG = '0';
            COST_TYPE=1;
            output;
            CURRENT_COST_FLG = '0';
            COST_TYPE=2;
            output;
         end;
         else if first.GEO_HIER_SK and not last.GEO_HIER_SK and COST_TYPE=1 then 
         do;
            output;
            obsnum = _n_ + 1;
            set GPE_WORK.GP_EXCEPTION_&part_sk._tmp(keep = cost_type 
                             rename = (cost_type = next_cost_type)) point = obsnum;
            if next_cost_type ne 2 then 
            do; 
               CURRENT_COST_FLG = '0';
               COST_TYPE=2;
               output;
            end;
         end;
         else if first.GEO_HIER_SK and not last.GEO_HIER_SK and COST_TYPE=2 then
         do;
            output;
            CURRENT_COST_FLG = '0';
            COST_TYPE=1;
            output;
         end;
         else output;
      %end;
      %else %do;
         if first.GEO_HIER_SK and last.GEO_HIER_SK and COST_TYPE in (2,3) then 
         do;
           output;
           CURRENT_COST_FLG = '0';
           COST_TYPE=1;
           output;
         end;
		 else output;         
      %end;            
   run;

   proc sql;
      create table GPE_WORK.GEO_PROD_EXCEPTION_&part_sk as
      select a.*, 
             case  
                when coalesce(geo.closing_dt,'31DEC5999'd) < "&src_ref_dt"d then '1'
                else '0'
             end as GEO_CLOSE_FLG length=1
        from GPE_WORK.GP_EXCEPTION_&part_sk._tmp a,
             DI_DM.geography_dm geo
       where a.geo_hier_sk = geo.geo_hier_sk
         and (PROD_DISCONTINUED_FLG = '1' or 
              INVENTORY_FLG         = '0' or
              CURRENT_PRICE_FLG     = '0' or
              CURRENT_COST_FLG      = '0' or
              coalesce(geo.closing_dt,'31DEC5999'd) < "&src_ref_dt"d )
      ;
   quit;
                 
   %no_process_exit:

%mend di_etl_geo_prod_exception;

/*
%di_etl_geo_prod_exception(
                        di_dm_lib_path=E:\tests_mini,
                        part_sk=2,
                        work_lib_path=%sysfunc(pathname(work)), 
                        prev_current_date_sk=20704,
                        src_ref_dt = 01AUG2009,
                        active_prod_status=%str(1,3,5,6,7,8),
                        use_landed_cost=0);
*/
