
/* 
SUPPORT: sinprs Swain, Pradeep
*/ 
/****************************************************************************************
                               DEMAND INTELLIGENCE
NAME        : di_etl_hp_migration
DESCRIPTION : This is the Raper Macro which calls All other Macros required for 
              Migration of data from 4.2 to 4.3 (4.2 Datamart --> HP)
              This Macro will run only for initial load (once), as data will be in 4.2 Format
              and this macro will populate partitions as required for HP.
INPUTS      : Please check the details in %di_etl_partition_data Macro.
OUTPUTS     : Partition datasets/tables
GLOBAL VAR  : PROD_PARTITION_LVL
USAGE       : %di_etl_hp_migration(from_lib=DI_DM,
                                   to_partition_path=D:\TK_test\_dm\111,
                                   exclude_list=,
                                   records_to_process=,
                                   increment_count=2000000,
                                   num_parallel_jobs=8, 
                                   refresh_mapping=1,
                                   resume=0, 
                                   override_parallel_failure=);
AUTHOR      : sinprs , sinsoj
SUPPORT     : sinprs Swain, Pradeep sinsoj Jain, Sourabh

HISTORY     : 16Jun2010 created
              15Mar2011 sinprs Removed conditions on table_nm to check inventory, sales and Promotion.
                               These partition jobs should run by default.
              21Mar2011 sinprs Added partition_method. Default=2 (small/medium)
                               If 1: use di_etl_partition_data(). -- for large datamart. > 10GB
                               If 2: use di_etl_subset_datamart().-- for small/med datamart <= 10GB

****************************************************************************************/
%macro di_etl_hp_migration( from_lib=,
                            to_partition_path=,
                            exclude_list=,
                            records_to_process=,
                            increment_count=,
                            num_parallel_jobs=, 
                            refresh_mapping=1,
                            resume=0, 
                            override_parallel_failure=,
                            partition_method=2);

   %let from_lib = %upcase(&from_lib);

   data _null_;
   pid  = &SYSJOBID;
   put '************************************************************************';
   put '*   Attention: The Process ID (PID) of this SAS Session is: ' pid ;
   put '************************************************************************';
   put '_';
   run;

   %local etls_startTime abort tab_list_vars;
   
   %let etls_startTime=%sysfunc(datetime(), datetime21.);
   
   **********************************************************************************;
   *** Start check macro parameters;
   **********************************************************************************; 
   %let syscc = 0;   
   %let abort = 0;
   
   %if %di_util_check_sym(sym=from_lib) > 0 %then %do;
      %put ERROR: ARG FROM_LIB IS NOT SPECIFIED;
      %let abort = 1;
   %end;
   
   %if %sysfunc(libref(&from_lib)) ne 0 %then %do;
      %put ERROR: LIBNAME &FROM_LIB DOES NOT EXIST;
      %let abort = 2;
   %end;

   %if %di_util_check_sym(sym=to_partition_path) > 0 %then %do;/*value not specified*/
      %put ______ TO_PARTITION_PATH IS NOT SPECIFIED, taking FROM_LIB/1 path as value;
   %end;
   %else %do;
      libname check "&to_partition_path";
      %if %sysfunc(libref(check)) ne 0 %then %do;
         %put ERROR: TO_PARTITION_PATH folder DOES NOT EXIST;
         %let abort = 3;
      %end;
   %end;

   %if &resume ne != '0' or &resume != '1' %then %do;
      %put ERROR: ARG RESUME should be 0 or 1;
      %let abort = 6;
   %end;

   %if &refresh_mapping ne %str(1) %then %do;
      %let refresh_mapping = 0;
      %put ______ Do not refresh mapping tables;
   %end;
   
   %if %di_util_check_sym(sym=num_parallel_jobs) > 0 %then %do;
      %put WARNING: ARG NUM_PARALLEL_JOBS IS NOT SPECIFIED. Global_setting will be used;

      * Fetch from Global_setting;
      %di_util_global_setting_spec(spec_nm=NUM_PARALLEL_EXECUTIONS, 
                                   app=ALL,
                                   _spec_value=num_parallel_jobs);
   %end;

   %if %di_util_check_sym(sym=records_to_process) > 0 %then %do;
      %put ______ Arg RECORDS_TO_PROCESS is not specified. ;
      %put ______ Will proceded with all Records.;
   %end;

   %if %di_util_check_sym(sym=increment_count) > 0 %then %do;
      %put ______ Arg INCREMENT_COUNT is not specified. ;
      %put ______ Will proceded for 2 Million records;
   %end;

   %if &abort gt 0 %then %do;
      %put ______ abort = &abort;
      %goto ERROR;
   %end;   

   **********************************************************************************;
   *** Check partition level in Global_setting;
   **********************************************************************************; 

   %local prod_partition_lvl;

   %di_util_global_setting_spec(di_data=&from_lib,
                                spec_nm=PROD_PARTITION_LVL, 
                                app=ALL,
                                _spec_value=prod_partition_lvl);

   %if &prod_partition_lvl eq %str() or &prod_partition_lvl lt 0 %then %do;
      %put ERROR: PROD_PARTITION_LVL is not defined in Global_setting;
      %goto ERROR;
   %end;
   %else %do;   
      %put ______ Partition level: &prod_partition_lvl;
   %end; 

   **********************************************************************************; 
   *** Create sub folders;
   **********************************************************************************; 

   %local from_lib_path list_part_sk num_parts_sk num_parts_sk
      sub_f idx;
   %global di_dm_lib_path;   
    
   %let from_lib_path = ;
   %let di_dm_lib_path =;

   proc sql noprint;
      select path into :from_lib_path
        from sashelp.vlibnam
       where upcase(libname) = upcase("&FROM_LIB")
         and level in (0,1);
   quit;
   
   %let di_dm_lib_path = &from_lib_path;

   %if %di_util_check_sym(sym=to_partition_path) > 0 %then %do;/*value not specified*/
   
      ** assign subdirectory 1 for partition base directory; 
      %let to_partition_path = &di_dm_lib_path/1;
      
   %end;
   
   %put ______ Partition Base: &to_partition_path;

   **********************************************************************************; 
   *** Set Increment Count;
   **********************************************************************************; 
   %if %di_util_check_sym(sym=increment_count) > 0 %then %do;
      %let increment_count = 2000000;
   %end;
   %else %do;
      %let increment_count = &increment_count;
   %end;
   %let syscc = 0;

   %if &partition_method = 1 %then %do;
   
      %di_etl_partition_data(from_lib=&from_lib,
                              to_partition_path=&to_partition_path,
                              exclude_list=&exclude_list,
                              records_to_process=&records_to_process,
                              increment_count=&increment_count,
                              num_parallel_jobs=&num_parallel_jobs, 
                              refresh_mapping=&refresh_mapping,
                              resume=&resume, 
                              override_parallel_failure=&override_parallel_failure);
   %end;
   %else %if &partition_method = 2 %then %do;
   
      %di_etl_subset_datamart(from_lib=&from_lib,
                              to_partition_path=&to_partition_path,
                              exclude_list=&exclude_list,
                              refresh_mapping=1,
                              num_parallel_jobs=&num_parallel_jobs, 
                              resume=&resume, 
                              override_parallel_failure=&override_parallel_failure);

                            
      
   %end;

   %if &syscc > 4 %then %do;
       %put ERROR: Error While Partitioning tables. exiting..;
       %goto ERROR;
   %end;   

   **********************************************************************************; 
   *** run only Promotion tables;
   **********************************************************************************; 
   %put _________in promotion fact partition;
   %di_etl_partition_promotion(from_lib=&from_lib, 
                             num_parallel_jobs=&num_parallel_jobs, 
                             to_partition_path=&to_partition_path);

   %if &syscc > 4 %then %do;
     %put ERROR: Error While Partitioning Promotion Tables. exiting..;
     %goto ERROR;
   %end;

   **********************************************************************************; 
   *** run only Inventory fact;
   **********************************************************************************;

   %if &partition_method = 1 %then %do;
   
      proc sql noprint;
          select min_value,max_value
          into :inv_min_value,:inv_max_value
          from(   select min(substr(memname,16,10)) as min_value,
                  max(substr(memname,16,10)) as max_value
                  from dictionary.tables
                  where upper(libname) = upper("&FROM_LIB")
                  and upper(memname) like 'INVENTORY_FACT%'
                  and upper(memname) ne 'INVENTORY_FACT_DATE_SK'
                and anydigit(substr(memname,16,10)) > 0
          );
      quit;

      %let inv_min_value = &inv_min_value;
      %let inv_max_value = &inv_max_value;
      %put _________inv_min_value = &inv_min_value;
      %put _________inv_max_value = &inv_max_value;

      %if &syscc > 4 %then %do;
         %put ERROR: Error While extracting DS for INVENTORY_FACT. exiting..;
         %goto ERROR;
      %end;

      %if %eval(&inv_min_value gt 0) %then %do;

         %di_etl_partition_inv_sales(from_dt_sk=&inv_min_value,
                          to_dt_sk=&inv_max_value,
                          table_nm=INVENTORY_FACT,
                          increment_count=&increment_count,
                          from_lib_path=&from_lib_path,
                          to_partition_path=&to_partition_path,
                          num_parallel_jobs=&num_parallel_jobs);
                          
      %end;   

   %end;
   %else %if &partition_method = 2 %then %do;
   
      %di_etl_subset_sales_inv_fact(
         dsn=INVENTORY_FACT,
         from_lib_path=&from_lib_path,
         to_lib_path=&to_partition_path);
      
   %end;
   
   %if &syscc > 4 %then %do;
     %put ERROR: Error While Partitioning Inventory_fact tables. exiting..;
     %goto ERROR;
   %end;   

   **********************************************************************************; 
   *** run only Sales fact;
   **********************************************************************************;

   %if &partition_method = 1 %then %do;
   
      proc sql noprint;
       select min_value,max_value
       into :sale_min_value,:sale_max_value
       from(   select min(substr(memname,12,10)) as min_value,
               max(substr(memname,12,10)) as max_value
               from dictionary.tables
               where upper(libname) = upper("&FROM_LIB")
               and upper(memname) like 'SALES_FACT%'
               and upper(memname) ne 'SALES_FACT_DATE_SK'
             and anydigit(substr(memname,12,10)) > 0
       );
      quit;

      %let sale_min_value = &sale_min_value;
      %let sale_max_value = &sale_max_value;
      %put ________sale_min_value = &sale_min_value;
      %put ________sale_max_value = &sale_max_value;

      %if &syscc > 4 %then %do;
       %put ERROR: Error While extracting DS for SALES_FACT. exiting..;
       %goto ERROR;
      %end;

      %if %eval(&sale_min_value gt 0) %then %do;

          %di_etl_partition_inv_sales(from_dt_sk=&sale_min_value,
                                      to_dt_sk=&sale_max_value,
                                      table_nm=SALES_FACT,
                                      increment_count=&increment_count,
                                      from_lib_path=&from_lib_path,
                                      to_partition_path=&to_partition_path,
                                      num_parallel_jobs=&num_parallel_jobs);
      %end;          
    
   %end;    
   %else %if &partition_method = 2 %then %do;
       
      %di_etl_subset_sales_inv_fact(
         dsn=SALES_FACT,
         from_lib_path=&from_lib_path,
         to_lib_path=&to_partition_path);

   %end;

   %if &syscc > 4 %then %do;
     %put ERROR: Error While Partitioning Sales_fact tables. exiting..;
     %goto ERROR;
   %end;

   %goto EXIT;
 
   %ERROR:
       %let syscc=5000;    
   %EXIT:

   %let etls_endTime=%sysfunc(datetime(), datetime21.);
 
   %put ______ Total Start Time : &etls_startTime;
   %put ______ Total End Time   : &etls_endTime;

   data _null_;
      s = input("&etls_startTime",datetime21.);
      e = input("&etls_endTime",datetime21.);
      d = round((e - s) / 60, 0.01);
      put "______ Grand Total time (minutes) : " d;
   run;

%mend di_etl_hp_migration;
