/* 
SUPPORT: sinprs Swain, Pradeep
*/ 

%macro di_etl_merge_and_sort(table_nm=, 
                      parent_dir=, 
                      part_list=, 
                      num_parts=,
                      l_prefix=,
                      from_lib=di_dm);

   data _null_;
      pid  = &SYSJOBID;
      put '************************************************************************';
      put '*   Attention: The Process ID (PID) of this SAS Session is: ' pid ;
      put '************************************************************************';
      put '_';
   run;
   
   %local all_input_tables all_tables_ds variable_list contains_prod_hier_sk contains_geo_hier_sk;

   %let partition_path = %di_util_create_folder(relative_dir=0,parent_dir=&parent_dir);
   libname PART_0  "&partition_path" %di_dio_libname;

   %let highlvl_path = %di_util_create_folder(relative_dir=highlvl,parent_dir=&partition_path);
   libname highlvl  "&highlvl_path" %di_dio_libname;


   %do x=1 %to %eval(&num_parts);  
      %let all_input_tables = ;
      %let all_tables_ds = ;
      %let tmp_part = %scan(&part_list,&x,%str(,));
      %let sub_f =&tmp_part;
      %if %eval(&sub_f lt 0) %then %do;
         %let tmp_part_sk=%sysfunc(abs(&tmp_part));
         %let sub_f =/n&tmp_part_sk;
      %end;
      %else %do;
         %let tmp_part_sk=&tmp_part;
         %let sub_f =/&tmp_part_sk;
      %end;
      %let tmp_part_sk = %sysfunc(putn(&tmp_part_sk,hex7.));
      %let partition_path = %di_util_create_folder(relative_dir=&sub_f,parent_dir=&parent_dir); 
      libname d&tmp_part_sk  "&partition_path" %di_dio_libname;
      
      %local tab_length;
      %let tab_length = %length(&table_nm);
      %let tab_length = %eval(&tab_length + 1);

      proc sql noprint;
         select cats(libname,".",memname), memname
          into :all_input_tables separated by ' ',
               :all_tables_ds separated by ' '
          from dictionary.tables
         where libname = "D&tmp_part_sk"
           and memname contains upcase("&table_nm._")
           and anydigit(memname,&tab_length) = &tab_length + 1;
      quit;

      %put ______ table_nm: &FROM_LIB..&table_nm;
      
      %let variable_list=%di_util_column_names(&FROM_LIB..&table_nm);
      
      * check for PROD_HIER_SK match;
      %if %sysfunc(find(&Variable_list,PROD_HIER_SK)) > 0 %then %do;
         %let contains_prod_hier_sk = 1;
      %end;
      * check for GEO_HIER_SK match;
      %if %sysfunc(find(&Variable_list, GEO_HIER_SK)) > 0 %then %do;
         %let contains_geo_hier_sk=1;
      %end;

      %if %length(&all_input_tables) gt 0 %then %do;
      
         data d&tmp_part_sk..&table_nm;
            set &all_input_tables highlvl.&table_nm;
         run;

         proc datasets lib = d&tmp_part_sk nolist nodetails; 
            delete &all_tables_ds;
         quit;

      %end;
      %else %do;
      
         /* just append the high level data into partition */
         data d&tmp_part_sk..&table_nm;
            set d&tmp_part_sk..&table_nm highlvl.&table_nm;
         run;
      %end;
         
      %local sorted_by sort_option;
      %if &contains_prod_hier_sk = 1 %then %do;
         %let sorted_by = PROD_HIER_SK;
      %end;
      %if &contains_geo_hier_sk = 1 %then %do;
         %let sorted_by = &sorted_by GEO_HIER_SK;
      %end;

      %if %upcase(&table_nm) = COST_FACT_HISTORY %then %do;
         %let sorted_by = &sorted_by COST_TYPE VALID_FROM_DT;
      %end;
      %else %if %upcase(&table_nm) = COST_FACT_CURRENT %then %do;
         %let sorted_by = &sorted_by COST_TYPE;
      %end;
      %else %if %upcase(&table_nm) = COST_FACT_FUTURE %then %do;
         %let sorted_by = &sorted_by COST_TYPE VALID_FROM_DT;
      %end;
      %else %if %upcase(&table_nm) = PRICE_FACT_HISTORY %then %do;
         %let sorted_by = &sorted_by VALID_FROM_DT;
      %end;
      %else %if %upcase(&table_nm) = PRICE_FACT_FUTURE %then %do;
         %let sorted_by = &sorted_by VALID_FROM_DT;
      %end;
      %else %if %upcase(&table_nm) = PROMOTION_FACT %then %do;
         %let sorted_by = &sorted_by VEHICLE_SK CONDITION_TYPE RANGE_START_DISC_PCT RANGE_END_DISC_PCT START_DT;
      %end;      
      %else %if %upcase(&table_nm) = GEO_PROD_STATUS %then %do;
               %let sorted_by = &sorted_by VALID_FROM_DT;
      %end;
      %else %if %upcase(&table_nm) = FUTURE_RECEIPT %then %do;
               %let sorted_by = &sorted_by RECEIPT_DT;
      %end;
      %else %if %upcase(&table_nm) = RETURN_FACT %then %do;
               %let sorted_by = &sorted_by DATE_SK;
      %end;
      %else %if %upcase(&table_nm) = GEO_PROD_OWNED_INV %then %do;
               %let sorted_by = &sorted_by DATE_SK;
      %end;
      %else %if %upcase(&table_nm) = DOW_SALES_FACT %then %do;
               %let sorted_by = &sorted_by DATE_SK DAY_OF_WEEK;
      %end;
      %else %if %upcase(&table_nm) = DOW_EVENT_SALES_FACT %then %do;
               %let sorted_by = &sorted_by DATE_SK DAY_OF_WEEK;
      %end;
      %else %if %upcase(&table_nm) = PROMO_SALES_FACT %then %do;
               %let sorted_by = &sorted_by START_DT END_DT;
      %end;
      %else %if %upcase(&table_nm) = PRODUCT_DM %then %do;
         %let sort_option = NODUPKEY;
      %end;
               
      proc sort data=d&tmp_part_sk..&table_nm &sort_option;
         by &sorted_by;
      run;
 
      proc datasets library=d&tmp_part_sk nolist;
         modify &table_nm;
         index create prod_hier_sk;
      quit;
      
   %end;
   
%mend;

/*%di_etl_merge_and_sort(table_nm=ATTR_PROD_DM, parent_dir=C:\tests\111, part_list=%str(2,9230,21194,52965), num_parts=4);*/
