/* 
SUPPORT: sinprs Swain, Pradeep
*/ 
%macro di_etl_partition_inv_sales(
             from_dt_sk=,
             to_dt_sk=,
             table_nm=,
             increment_count=,
             from_lib_path=,
             to_partition_path=,
             num_parallel_jobs=
             );

   %local log_prefix;
   
   %let table_nm = %upcase(&table_nm);
   
   %if "&table_nm" eq "SALES_FACT" %then %do;
      %let log_prefix = SALES;
   %end;
   %else %if "&table_nm" eq "INVENTORY_FACT" %then %do;
      %let log_prefix = INV;
   %end;
   %else %do;
      %let syscc = 5000;
      %put ERROR: table_nm must be either SALES_FACT or INVENTORY_FACT. Exit;
      %return;
   %end;

   **********************************************************************************;
   *** Creating Empty Datasets;
   **********************************************************************************; 
   libname from_lib "&from_lib_path";
   proc sql noprint;
       select part_sk, count(part_sk)
       into :part_list separated by ' ', :num_parts
       from di_mon.distinct_part_sk;
   quit;

   %do x=1 %to %eval(&num_parts);  /* create the subset folders and assign libraries */
      %let tmp_part = %scan(&part_list,&x,%str( ));
      %let sub_f =&tmp_part;
      %let partition_path = %di_util_create_folder(relative_dir=&sub_f,parent_dir=&to_partition_path); 
      libname d_temp  "&partition_path" %di_dio_libname;

      /*creating empty DS*/
      data d_temp.&table_nm;
      set from_lib.&table_nm;
      stop;
      run;
   %end;
      
   data &table_nm._list;
      attrib from_dt_sk length=4 to_dt_sk length=4 incr_count length=4;
      do idx=&from_dt_sk to &to_dt_sk;
         _input=cats("&table_nm");
         _output=cats("&table_nm");
         from_dt_sk=idx;
         to_dt_sk=idx;
         incr_count=&increment_count;
         from_lib_path="&from_lib_path";
         to_lib_path="&to_partition_path";
         output;
      end;
      drop idx;
   run;

   %di_util_mp_connect(mac_name=di_etl_partition_inv_sales_unit,
                       arg_table= &table_nm._list,
                       nm_prll_exec=&num_parallel_jobs,
                       prefix=&log_prefix);
                       
   data Child_syscc_&table_nm.;
                  set m_con.Child_syscc_status;
   run;

   %local num_parts_per_proc;

   %let num_parts_per_proc = %eval(%di_util_nobs(di_mon.distinct_part_sk)/&num_parallel_jobs);
   %let num_parts_per_proc = %eval(&num_parts_per_proc + 1);

   data merge_inv_sales_partitions;
      attrib parent_dir length=$200 part_list length=$2000 num_parts length=4 rem length=4;
      retain part_list '' num_parts 0;
      dsn="&table_nm";
      parent_dir = "&to_partition_path";
      set di_mon.distinct_part_sk end=eof;
      rem=mod(_n_,&num_parts_per_proc);
      num_parts = num_parts+1;
      part_list = catx(",",part_list,cats(part_sk));
      if (rem eq 0 or eof) then do;
         output;
         num_parts = 0; part_list = '';
      end;
      drop rem part_sk;
   run;

   %di_util_mp_connect(mac_name=merge_inv_sales_fact,
                    arg_table=merge_inv_sales_partitions,
                    nm_prll_exec=&num_parallel_jobs,
                    prefix=&log_prefix);
   data Child_syscc_&table_nm._1;
                     set m_con.Child_syscc_status;
   run;

%mend di_etl_partition_inv_sales;


/*

%di_etl_partition_inv_sales(from_dt_sk=20601,
       to_dt_sk=20704,
       table_nm=SALES_FACT,
       increment_count=10000000,
       from_lib_path=/mst/warehouse/tests_bulked,
       to_partition_path=/mst/warehouse/tests_bulked/script/inv_fact,
       num_parallel_jobs=24);

%di_etl_partition_inv_sales(from_dt_sk=20601,
       to_dt_sk=20704,
       table_nm=INVENTORY_FACT,
       increment_count=8000000,
       from_lib_path=/mst/warehouse/tests_bulked,
       to_partition_path=/mst/warehouse/tests_bulked/script/inv_fact,
       num_parallel_jobs=24);


*/
