/* 
SUPPORT: sinprs Swain, Pradeep
*/ 
%macro di_etl_partition_inv_sales_unit(
         from_lib_path=,
         from_dt_sk=,
         to_dt_sk=,
         _output=,
         to_lib_path=,
         _input=, 
         incr_count=8000000);

 data _null_;
    pid  = &SYSJOBID;
    put '************************************************************************';
    put '*   Attention: The Process ID (PID) of this SAS Session is: ' pid ;
    put '************************************************************************';
    put '_';
 run;
 
   %local first_row last_row incr_count ds_obs part_list num_parts data_var_list 
      table_list tmp_part sub_f tmp_part_sk counter start_time datesk;

   data _null_;
      a=datetime();
      call symputx('start_time', trim(put(a,datetime21.)), 'l');
   run;

   libname from_lib "&from_lib_path" %di_dio_libname;

   proc sql noprint;
       select part_sk, count(part_sk)
       into :part_list separated by ' ', :num_parts
       from di_mon.distinct_part_sk;
   quit;

   /* create libraries */
   %do x=1 %to %eval(&num_parts);  /* create the subset folders and assign libraries */
      %let tmp_part = %scan(&part_list,&x,%str( ));
      %let sub_f =&tmp_part;
      %if &sub_f lt 0 %then %do;
         %let tmp_part_sk=%sysfunc(abs(&tmp_part));
         %let sub_f =/n&tmp_part_sk;
      %end;
      %else %do;
         %let tmp_part_sk=&tmp_part;
         %let sub_f =/&tmp_part_sk;
      %end;
      %let tmp_part_sk = %sysfunc(putn(&tmp_part_sk,hex7.));
      %let partition_path = %di_util_create_folder(relative_dir=&sub_f,parent_dir=&to_lib_path); 
      libname d&tmp_part_sk  "&partition_path" %di_dio_libname;

   %end;

   %do datesk=&from_dt_sk %to &to_dt_sk; /* Start:re-partition each partition*/
      %let dataset =%str(&_input)_&datesk;
      %*let source_records= %eval(&source_records + %di_util_nobs(from_lib.&dataset)) ;
      %if (%di_util_nobs(from_lib.&dataset) > 0) %then %do;
         data &dataset._vw/view=&dataset._vw;
            set from_lib.&dataset; attrib DATE_SK length=6;
            DATE_SK = &datesk;
         run;
         %let table_list = &table_list %str(&dataset._vw);
      %end;
   %end;    

   data &_output._part_sk;
      set &table_list;
      attrib PART_SK length=6;
      retain init_hash 0;
      drop init_hash hash_prod_hier_sk rc0;
      attrib hash_prod_hier_sk length=6 date_sk length=6;
      if (not init_hash) then do;
         init_hash = 1;
         declare hash prod_filter(hashexp: 16, dataset: "di_mon.hash_prod_filter_table");
         prod_filter.defineKey('hash_prod_hier_sk');
          prod_filter.defineData('part_sk');
         prod_filter.defineDone();
      end;
      hash_prod_hier_sk=prod_hier_sk;
      rc0 = prod_filter.find();
   run;

   proc sort data=&_output._part_sk %di_dio_sort;
      by part_sk;
   run;

   %di_util_get_table_columns(ds=&di_data..&_output, _columns=data_var_list);

   %let data_var_list = &data_var_list;
   %put ______ Variable list = &data_var_list;

   %let first_row=1;
   %let last_row=0;
   %let ds_obs = %di_util_nobs(work.&_output._part_sk);
   %put ______ ds_obs = &ds_obs;
   %let counter = 0;

   %put ______ first_row = &first_row last_row = &last_row counter = &counter;
   %if &ds_obs > 0 %then %do;
      %do %while (&last_row < &ds_obs);
         %let first_row = %eval(&last_row+1);
         %let last_row=%eval(&last_row+&incr_count);
         %if (&last_row > &ds_obs) %then %do;
            %let last_row = &ds_obs;
         %end;
         %let counter = %eval(&counter + 1);

         data _null_;
            if 0 then set &_output._part_sk(firstobs=&first_row obs=&last_row);
            attrib libref length=$9;
            dcl hash partition ( ordered: 'a');
            partition.definekey ('PROD_HIER_SK','GEO_HIER_SK','_n_'); 
            partition.definedata (&data_var_list);
            partition.definedone ();
            do _n_ = 1 by 1 until (last.PART_SK);
               set &_output._part_sk(firstobs=&first_row obs=&last_row);
               by PART_SK;
               partition.add();
            end;
            libref= cats("d",put(part_sk,hex7.),".");
            partition.output (dataset: compress(libref||"&_output._&to_dt_sk"||"_&counter"));
         run;
      %end;
   %end;

   proc datasets lib = work nolist nowarn memtype = (data view);
       delete &_output._part_sk ;  
   quit;

   data _null_;
      a=put(datetime(),datetime21.);
      put '______ Start time :' "&start_time";
      put '______ End time   :' a;
   run;

%mend;


/*option msglevel=i nofullstimer nosymbolgen mprint;*/
/*%di_etl_partition_inv_sales_unit(from_lib_path=\\puneasd10\MI32\Foundation\DI_DM\tests,*/
/*                      from_dt_sk=20601,*/
/*                      to_dt_sk=20601,*/
/*                      _output=SALES_FACT,*/
/*                      to_lib_path=\\puneasd10\MI32\Foundation\DI_DM\tests\111,*/
/*                      _input=SALES_FACT,*/
/*                      incr_count=800);*/
