/****************************************************************************************
                               DEMAND INTELLIGENCE

NAME:          di_util_custom_sort_format

DESCRIPTION:   Macro to add custom format to the Size Profiling Extract. 

APPLICATIONS:  SO

AUTHOR:         sinsks Das, Sourav

SUPPORT:        sinsks Das, Sourav

HISTORY:        18Jan2017, Sourav Started creating the macro
				29Mar2018  Sourav F4MICM54 Use _N_ to create the formats 
****************************************************************************************/

%macro di_util_custom_sort_format(project_sk=);

%local start_time end_time not_empty_input;

   %di_util_get_time(_time = start_time);

   /*** Formats Location Library assignment ***/
   
   %let format_location = %str();
   
   proc sql noprint;
      select param_value into :format_location 
         from di_mon.job_param
            where param_name="FORMAT_LOCATION" 
               and jobname="DI_UTIL_CUSTOM_SORT_FORMAT";
   quit;
            
   %let format_location = &format_location;
   %put Location of Formats Catalog = &format_location;
   
   %if NOT %length(&format_location) gt 0 %then
   %do; 
      %put ERROR: Location of Format library is not mentioned. Please add the value in Job_Param table for DI_UTIL_CUSTOM_SORT_FORMAT ;
      %goto exit;
   %end;
   
   libname fmt_loc "&format_location.";
   
   /*** Check the rows of di_trans.std_size_cd ***/
   
    proc sql noprint;
      select count(*) into :cnt from di_trans.std_size_cd;
   quit;

   %if not &cnt. gt 0 %then
      %do;
         %put ERROR: No data in DI_TRANS.STD_SIZE_CD, Skipping Custom Sort.;
         %goto exit;
      %end;
   

   proc sort data=di_trans.std_size_cd out=work.tmp_std_size_cd;
   by display_order;
   quit;

   data work.tmp_std_size_cd;
   set work.tmp_std_size_cd;
   tmp = input(compress(_N_||display_order),8.);
   run; 


   /*** Create the user-defined format ***/

         DATA ctrl;
          SET tmp_std_size_cd;
          RETAIN fmtname 'szpf' type 'N';
          RENAME tmp = Start;
          LABEL = std_size_nm;
         RUN; 

         proc format library=work cntlin=ctrl;
         quit;
         proc format library=fmt_loc cntlin=ctrl;
         quit;
    

   %if %length(&project_sk) gt 0 %then
      %do;
      
      proc sql noprint;
      select path into :data_lib_path
      from sashelp.vlibnam
      where libname = "DI_DATA"
      and level in (0,1);
      quit;

      %put ______: data_lib_path=&data_lib_path;
      %let rdm_path = %di_util_create_folder(relative_dir= so_reports , parent_dir = &data_lib_path);
      %put ______: rdm_path = &rdm_path;

      libname rpt_so "&rdm_path.";

         %if not %di_util_nobs(rpt_so.szpf_extract_&project_sk.) gt 0 %then
            %do;
               %put ERROR: No data in rpt_so.szpf_extract_&project_sk or table does not exist, Skipping Custom Sort.;
               %goto exit;
            %end;
      
         /*** Apply the format in the report extract ***/

         
      proc sql noprint;
      create table rpt_so.szpf_extract_&project_sk. as 
      select A.profiling_project_sk, A.project_nm, A.start_dt, A.end_dt, A.updated_dttm, A.app_user_id, A.season_nm, A.prf_scenario_sk, A.scenario_nm,
      A.store_category_nm, A.product_root_nm, A.project_attribute_filter, A.scenario_status, A.prod_partition_nm, A.sizeset_id, A.Sizeset_nms, A.Sizeset_cds,
      A.size_set_type, A.profile_sk, A.profile_nm, A.manual_flg, A.profile_prod_sk, A.product, A.Prod_hier_SK, A.prod_id, A.prod_nm, A.store_group_id, A.store_group_nm,
      A.geo_hier_sk, A.geo_id, A.geo_nm, A.street_address, A.city, A.state, A.zip_code, A.country, A.assignment_type, A.std_size_cd, A.display_order,
      A.revenue, A.sales_units, A.Inventory_eop_units, A.imputation_added_sales_units, A.post_imputation_sales_units, A.In_stock_Opportunity, A.Period_in_stock,
      A.profile_pct, A.prelim_profile_rmsd, B.tmp as std_size_nm format=SZPF.
       from WORK.TMP_STD_SIZE_CD B, RPT_SO.SZPF_EXTRACT_&project_sk. A
      where A.std_size_cd = B.STD_SIZE_CD;
      quit;

   %end; 
   %EXIT:
   
   %di_util_get_time(_time = end_time);
   %put NOTE: macro DI_UTIL_CUSTOM_SORT_FORMAT started at &start_time., ended at &end_time.;
   %put ______ Exiting from the macro DI_UTIL_CUSTOM_SORT_FORMAT....;

%mend;
