/* 
SUPPORT: xiyong Xiao, Yongqiao
*/ 
%macro di_util_partition_lib(
  di_data=,
  base_table=,
  path_search=0,
  create_dir=1,
  partition_lib=,
  partition_table=, /* if this file is specifed then all the concatenations of di_data are being 
                       searched to find the folder containing this file */
  reassign=1
  ) 
  /*storesecure*/;


%local di_path partition_path rc found_di_path lref nobs i;

%if (%length(&partition_table) = 0) and &path_search = 0 %then %do;
  proc sql noprint;
    select path into :di_path 
    from sashelp.vlibnam 
    where upcase(libname) = upcase("&di_data")
    and level in (0,1);
  quit;
  
  %let di_path = %qcmpres(&di_path);
  %di_util_assert_sym(sym=di_path);
  %let found_di_path = &di_path;
%end;
%else %do;

  %if "&reassign"="0" %then %do;
      %if %sysfunc(libref(&partition_lib))=0 %then %do;
          %goto EXIT;
      %end;
  %end;

  proc sql noprint;
    create table _di_path as 
    select DISTINCT path, level 
    from sashelp.vlibnam where upcase(libname) = upcase("&di_data")
    order by level;
  quit;

  %let lref = %sysfunc(open(_di_path));
  %let nobs = %sysfunc(attrn(&lref, NLOBS));

  %do i = 1 %to &nobs;
    %let rc = %sysfunc(fetchobs(&lref, &i));
    %let di_path = %qcmpres(%sysfunc(getvarc(&lref, 1)));
    %di_util_assert_sym(sym=di_path);
    
    %if (%length(&found_di_path) = 0) %then %let found_di_path = &di_path;
    
    %let base_table=%lowcase(&base_table);
    %let partition_path = &di_path/&base_table;
    %if (%sysfunc(fileexist(&partition_path))) %then %do;
      libname &partition_lib "&partition_path";
      %if &path_search = 1 %then %goto EXIT;
      %if (%sysfunc(exist(&partition_lib..&partition_table))) %then %do;
        %goto EXIT;
      %end;
    %end;
  %end;
  
%end;

%di_util_assert_sym(sym=found_di_path);

* assign partition_lib;
%put current libref(&partition_lib): %sysfunc(libref(&partition_lib));
%put di_path: &found_di_path;

*Convert table name to lower case;
%let base_table=%lowcase(&base_table);
%let partition_path = &found_di_path/&base_table;

%if "&reassign"="0" %then %do;
    %if %sysfunc(libref(&partition_lib))=0 %then %do;
        %goto EXIT;
    %end;
%end;

libname &partition_lib "&partition_path";

%put assigned libref(&partition_lib): %sysfunc(libref(&partition_lib));

*Check to see if partition_path exists and create it if not;
%if (not %sysfunc(fileexist(&partition_path)) and &create_dir=1) %then %do;
    %let rc = %sysfunc(dcreate(&base_table, &found_di_path));
%end;
  
%EXIT:

%if (%length(&lref) > 0) %then %do;
  %let rc = %sysfunc(close(&lref));
  proc sql;
    drop table _di_path;
  quit;
%end;

%mend di_util_partition_lib;

/*
libname di_data ("\\disrv5\Official Data Sets\RO3.2\test" "\\disrv5\Official Data Sets\RO3.2\mktdemo\model_data");
%di_util_partition_lib(
  di_data=di_data,
  base_table=model_data,
  partition_lib=p_md,
  partition_table=partition_2
  ) ;

*/
