/****************************************************************************************
                               DEMAND INTELLIGENCE

NAME:           di_util_purge_unit_by_gp

DESCRIPTION:    Macro to Purge GEO/PROD from library.
                This macro is called in different sessions using mp_connect.
                

APPLICATIONS:   All

MODULE:         util

OUTPUTS:        

AUTHOR:         sinsro Rokde, Sangeeta

SUPPORT:        sinsro Rokde, Sangeeta

HISTORY:        sinsro: SIRIUS Track 7611156181 -  DI_ETL_Geo_prod_purge_job runs long
                The SQL query to purge Prods and geos was taking too long [in hrs] to purge.
                The query is replaced with hash lookup to purge 

****************************************************************************************/
%macro gen_table_nm(memname=, purge_lib=);

/* This macro is used to generate the path of the table being purged*/
/* and this path is used in purge_result_table                      */

%let memname=&memname;
%let purge_lib=&purge_lib;

%local hx_part_sk orig_part_sk lib_prefix;
%let lib_prefix=;
%global purge_table_name;

%local path_info;
%let path_info = %di_util_first_libpath(DI_DM_P);
%put ______: DI_DATA path_info=&path_info;

%let hx_part_sk = %scan(&purge_lib,-1,_);
%let orig_part_sk = %sysfunc(inputn(&hx_part_sk,hex6.));
%put orig_part_sk = &orig_part_sk;

%let lib_prefix = %scan(&purge_lib,1,_);
%put lib_prefix=&lib_prefix;

%if &lib_prefix=P or &lib_prefix=D %then %do;
   %let purge_table_name=DI_DATA\1\&orig_part_sk\&memname;
%end;
%else %if &purge_lib = SUB_GPF %then %do;
   %let purge_table_name=DI_DATA\GEO_PROD_BENEFITS\&memname;
%end;
%else %if &purge_lib = SUB_GPFS %then %do;
   %let purge_table_name=DI_DATA\GEO_PROD_BENEFITS_PS\&memname;
%end;
%else %if &purge_lib = STR_EST %then %do;
   %let purge_table_name=DI_DATA\MPLN_STORE_EST_MTS\&memname;
%end;
%else %if &purge_lib = PRD_ACT %then %do;
   %let purge_table_name=DI_DATA\MPLN_PERIOD_ACTUAL_MTS\&memname;
%end;
%else %if &purge_lib = PRD_EST %then %do;
   %let purge_table_name=DI_DATA\MPLN_PERIOD_EST_MTS\&memname;
%end;
%else %if &purge_lib = PRD_HIST %then %do;
   %let purge_table_name=DI_DATA\MPLN_PERIOD_EST_MTS_HIST\&memname;
%end;
%else %if &purge_lib = SZ_PTCP %then %do;
   %let purge_table_name=DI_DATA\PROD_SIZE_PTCP\&memname;
%end;
%else %if &purge_lib = SZ_SET %then %do;
   %let purge_table_name=DI_DATA\PROD_SIZESET\&memname;
%end;
%else %if &purge_lib = GEO_SZ %then %do;
   %let purge_table_name=DI_DATA\PROD_GEO_SIZE\&memname;
%end;
%else %if &purge_lib = STR_EXL %then %do;
   %let purge_table_name=DI_DATA\STORE_EXCLUDED\&memname;
%end;
%else %if &purge_lib = STR_GPM %then %do;
   %let purge_table_name=DI_DATA\STORE_GRP_MBR\&memname;
%end;
%else %if &purge_lib = DI_DM2 or  &purge_lib = DI_TRANS %then %do;
   %let purge_table_name=DI_TRANS\&memname;
%end;
%else %if &purge_lib = DI_DM_P %then %do;
   %let purge_table_name=DI_DATA\&memname;
%end;
%else %if %symexist(etlp_incl_partition) and &etlp_incl_partition ne %str() %then %do; 
      %do i = 1 %to %eval(%sysfunc(countc(%bquote(&etlp_incl_partition),%STR(,)))+1);
         %let include&i=%qscan(%bquote(&etlp_incl_partition),&i,%str(,));
		 %if (not %sysfunc(fileexist(&path_info./&&include&i))) %then %do;
            %put ERROR: missing required folder: &path_info./&&include&i;
         %end;
         %else %do;
		    %if %upcase(&purge_lib)=%upcase(SUB&i) %then %do;
               %let purge_table_name=%upcase(DI_DATA\&&include&i\&memname);
			%end;
         %end;
      %end; 
%end;
%else %do;
   %let purge_table_name=&purge_lib\&memname;
%end;
     
%mend gen_table_nm;

%macro di_prod_delete_dangling_nodes(purge_lib =,
                                     memname=);
/*===================================================================================*/
/* Macro to delete dangling nodes from product_hier_assoc_dm                         */
/*===================================================================================*/

%put ______: Purging Product dangling nodes if any;

%local last_ds no_of_prod_assoc_cd 
       k max_prod_lvl lvl etl_msg;

%let last_ds = &purge_lib..&memname.;

proc sort data=&purge_lib..Product_lvl out=prod_lvl(keep=prod_lvl prod_hier_assoc_cd);
   by prod_hier_assoc_cd prod_lvl;
run;

data _null_;
   set prod_lvl;
   by prod_hier_assoc_cd prod_lvl;
      if last.prod_hier_assoc_cd then
      call symput(cats('g_max_prod_lvl_',prod_hier_assoc_cd),prod_lvl);
run;

proc sql noprint;
   select distinct prod_hier_assoc_cd into 
      :hier_assoc_cd1-:hier_assoc_cd100
   from &purge_lib..product_lvl;
quit;

proc datasets nodetails nolist nowarn lib=temp_lib;
      delete prod_with_dangling_nodes;
quit;

data temp_lib.prod_with_dangling_nodes;
   set temp_lib.Prod_purge_list;
run;

proc datasets LIBRARY=&purge_lib nolist noprint; 
   contents data = &memname OUT2=temp_lib.PHA_index; 
quit;

%let no_of_prod_assoc_cd = &sqlobs;

%put no_of_prod_assoc_cd = &no_of_prod_assoc_cd;

%do k=1 %to &no_of_prod_assoc_cd;

   %let max_prod_lvl = &&&&g_max_prod_lvl_&&hier_assoc_cd&k ;
   
   %do lvl=%eval(&max_prod_lvl) %to 1 %by -1;
      proc datasets nodetails nolist nowarn lib=work;
         delete _dangling_nodes;
      quit;
      proc sql noprint;
         create table work._dangling_nodes as 
         select prod_hier_assoc_cd length=4, prod_hier_sk length=4
         from (
         select distinct prod_hier_assoc_cd, prod_hier_sk
         from &last_ds.
         where prod_lvl eq %eval(&lvl-1)
         and   prod_hier_assoc_cd = &&hier_assoc_cd&k
         except 
         select distinct prod_hier_assoc_cd, parent_prod_hier_sk 
         from &last_ds.
         where prod_hier_sk ne parent_prod_hier_sk
         and   prod_lvl eq &lvl 
         and   prod_hier_assoc_cd = &&hier_assoc_cd&k);
      quit;

      proc sort data=work._dangling_nodes;
         by prod_hier_assoc_cd prod_hier_sk;
      run;
      %if &sqlobs gt 0 %then %do;
         %let etl_msg = %etl_msg(mid=5201,args=&sqlobs %eval(&lvl-1));
         %put &etl_msg;

            data &last_ds;
               merge &last_ds work._dangling_nodes(in=del);
               by prod_hier_assoc_cd prod_hier_sk;
               if not del;
            run;
            proc append data=work._dangling_nodes(drop=prod_hier_assoc_cd) 
                     base=temp_lib.prod_with_dangling_nodes force;
            run;
      %end;
   %end;

   proc datasets nodetails nolist nowarn lib=work;
      delete _dangling_nodes;
   quit;
%end;

   %etl_drop(&last_ds,ic);
   %etl_drop(&last_ds,index);

   data _null_;       
      set temp_lib.PHA_index end=eof; 
      where %upcase(Member)=%upcase("&memname") and %upcase(libname)=%upcase("&purge_lib");
      if _n_ eq 1 then call execute("proc datasets lib=&purge_lib nolist; modify &memname;"); 
      call execute("      " || recreate); 
      if eof then call execute("quit;");  
   run;
   
   proc datasets nodetails nolist nowarn lib=temp_lib;
      delete PHA_index;
   quit;


%mend di_prod_delete_dangling_nodes;

%macro di_geo_delete_dangling_nodes(purge_lib =,
                                    memname=);

/*===================================================================================*/
/* Macro to delete dangling nodes from geography_hier_assoc_dm                       */
/*===================================================================================*/


%put ___________ Purging Geography dangling nodes if any;

%local last_ds no_of_geo_assoc_cd 
       k max_geo_lvl lvl etl_msg;

%let last_ds = &purge_lib..&memname.;

proc sort data=&purge_lib..geography_lvl out=geo_lvl(keep=geo_lvl geo_hier_assoc_cd);
   by geo_hier_assoc_cd geo_lvl;
run;

data _null_;
   set geo_lvl;
   by geo_hier_assoc_cd geo_lvl;
      if last.geo_hier_assoc_cd then
      call symput(cats('g_max_geo_lvl_',geo_hier_assoc_cd),geo_lvl);
run;

proc sql noprint;
   select distinct geo_hier_assoc_cd into 
      :hier_assoc_cd1-:hier_assoc_cd20
   from &purge_lib..geography_lvl;
quit;

proc datasets nodetails nolist nowarn lib=temp_lib;
      delete geo_with_dangling_nodes;
quit;

data temp_lib.geo_with_dangling_nodes;
   set temp_lib.Geo_purge_list;
run;

proc datasets LIBRARY=&purge_lib nolist noprint; 
   contents data = &memname OUT2=temp_lib.GHA_index; 
quit;

%let no_of_geo_assoc_cd = &sqlobs;

%do k=1 %to &no_of_geo_assoc_cd;

   %let max_geo_lvl = &&&&g_max_geo_lvl_&&hier_assoc_cd&k ;
   
   %do lvl=%eval(&max_geo_lvl) %to 1 %by -1;
      proc datasets nodetails nolist nowarn lib=work;
         delete _dangling_nodes;
      quit;
      proc sql noprint;
         create table _dangling_nodes as 
         select geo_hier_assoc_cd length=4, geo_hier_sk length=4
         from (
         select distinct geo_hier_assoc_cd, geo_hier_sk
         from &last_ds.
         where geo_lvl eq %eval(&lvl-1)
         and   geo_hier_assoc_cd = &&hier_assoc_cd&k
         except 
         select distinct geo_hier_assoc_cd, parent_geo_hier_sk 
         from &last_ds.
         where geo_hier_sk ne parent_geo_hier_sk
         and   geo_lvl eq &lvl 
         and   geo_hier_assoc_cd = &&hier_assoc_cd&k);
      quit;

      proc sort data=_dangling_nodes;
         by geo_hier_assoc_cd geo_hier_sk;
      run;
      %if &sqlobs gt 0 %then %do;
         %let etl_msg = %etl_msg(mid=5201,args=&sqlobs %eval(&lvl-1));
         %put &etl_msg;
		 data &last_ds;
             merge &last_ds _dangling_nodes(in=del);
             by geo_hier_assoc_cd geo_hier_sk;
             if not del;
         run;
         proc append data=work._dangling_nodes(drop=geo_hier_assoc_cd) 
                     base=temp_lib.geo_with_dangling_nodes force;
         run;
     %end;
   %end;
   proc datasets nodetails nolist nowarn lib=work;
      delete _dangling_nodes;
   quit;

%end;

   %etl_drop(&last_ds,ic);
   %etl_drop(&last_ds,index);

   data _null_;       
      set temp_lib.GHA_index end=eof; 
      where %upcase(Member)=%upcase("&memname") and %upcase(libname)=%upcase("&purge_lib");
      if _n_ eq 1 then call execute("proc datasets lib=&purge_lib nolist; modify &memname;"); 
      call execute("      " || recreate); 
      if eof then call execute("quit;");  
   run;

   proc datasets nodetails nolist nowarn lib=temp_lib;
      delete GHA_index;
   quit;

%mend di_geo_delete_dangling_nodes ;
%macro purge_ind_tbl(
                                  table_to_purge=,
                                  purge_lib = ,
                                  geo_filter_table = ,
                                  prod_filter_table = ,
                                  purge_result_table =,
                                  ds_type=,
                                  temp_work_path=);

%put ______: executing purge_ind_tbl;
%put ______: table_to_purge=&table_to_purge;
%put ______: geo_filter_table=&geo_filter_table;
%put ______: prod_filter_table=&prod_filter_table;
%put ______: purge_result_table=&purge_result_table;
%put ______: ds_type=&ds_type;
%put ______: temp_work_path=&temp_work_path;
%put ______: purge_lib=&purge_lib;

/*Create concatenated library say di_dm_p*/
   %di_util_create_assign_libraries(create_lib=0);
/*define the work library*/
libname temp_lib "&temp_work_path";

%global partition_table_list;

   data work.partition_tables (keep=table_nm rename=(table_nm=memname));
      set di_trans.table_partition_meta;
      where ETL_STG2DM=1;
   run;

   proc sql noprint;
      select distinct(memname) into: partition_table_list separated by ' ' from work.Partition_tables
      where memname not in ('GEO_PROD_OVERRIDE','GEO_PROD_STATUS_OVERRIDE');
   quit;

%let partition_table_list=&partition_table_list;
%put ______ partition_table_list=&partition_table_list;

%local row_cnt ctr_arg k i act_part_sk;

/*===================================================================================
Get the count of observations in each arg table 
===================================================================================*/
proc sql noprint;
     select count(*) into: row_cnt from &table_to_purge;
quit;

%let row_cnt=&row_cnt;
%put ______: row_cnt in &table_to_purge = &row_cnt;

/*===================================================================================
Purge the tables one by one listed in arg table in temp_lib 
===================================================================================*/
%if &row_cnt gt 0 %then %do; /* Begin:Loop1*/
   %do k= 1 %to &row_cnt; /* Begin:Loop2*/
      %local init_cnt final_cnt;
      %let contains_geo_hier_sk = 0;
      %let contains_prod_hier_sk = 0;
      data _null_;
        pnt_var=&k;
         set &table_to_purge point=pnt_var;
         call symput('memname',trim(left(memname)));
         call symput('contains_prod_hier_sk',trim(left(prod_hier_sk_true)));
         call symput('contains_geo_hier_sk',trim(left(geo_hier_sk_true)));
         call symput('purge_lib',trim(left(libname)));
         stop;
      run;
     %let contains_prod_hier_sk=&contains_prod_hier_sk;
     %let contains_geo_hier_sk=&contains_geo_hier_sk;
     %let table_name=&memname;
     %let purge_lib=&purge_lib;
     %put contains_prod_hier_sk=&contains_prod_hier_sk;
     %put contains_geo_hier_sk=&contains_geo_hier_sk;
     %put table_name=&memname;
     %put purge_lib=&purge_lib;
     %put prod_hier_sk_true=&prod_hier_sk_true;
     %put geo_hier_sk_true=&geo_hier_sk_true;

   /*if the table to be purged belong to SAS then do the following*/
   %if %upcase(&ds_type)=%upcase(SAS) %then %do; 
       
      %let init_count = %di_util_nobs(&purge_lib..&table_name);
      %local proceed filter;
      %let filter = %str();
	  %let proceed=0;
/*================================================================================================================*/
/* Ref: customer track 7611156181 and Defect-S1094585: CRP:DI_ETL_Geo_prod_purge_job runs long                         */
/* a) New table (t1) is created having rows from &table_name minus rows to &prod_filter_table                     */
/*    the rows are deleted using hash                                                                             */
/* b) Truncate &table_name [the table to purge]                                                                   */
/* c) Append t1 into &table_name                                                                                  */
/* This was pretty faster and all indexes and constraints are maintained too                                      */
/*================================================================================================================*/		      

      * check for PROD_HIER_SK match;
      %if (&contains_prod_hier_sk = 1 and &prod_hier_sk_true = 1) %then %do;
         %let filter = &filter %str(rc_prod_filter = prod_filter.find();if rc_prod_filter = 0 then delete;);
         %put ______: filter = &filter;
         %let proceed=1;
      %end;

      * check for GEO_HIER_SK match;
      %if (&contains_geo_hier_sk = 1 and &geo_hier_sk_true = 1) %then %do;
         %let filter = &filter %str(rc_geo_filter = geo_filter.find();if rc_geo_filter = 0 then delete;);
         %put ______: filter = &filter;
		 %let proceed=1;
      %end;

	  %if &proceed eq 1 %then %do;
	     %put ______: Purging table:  &purge_lib..&table_name;
	     
         proc datasets lib=work nolist nowarn memtype = (data view);
            delete _&table_name ;
         quit;
	    
         data work._&table_name(keep = %di_util_column_names(&purge_lib..&table_name));
            set &purge_lib..&table_name;
            if _N_ =1 then do;
            %if (&contains_geo_hier_sk = 1 and &geo_hier_sk_true = 1) %then %do;
	       declare hash geo_filter(hashexp: 5, dataset:"&geo_filter_table");
               geo_filter.defineKey('geo_hier_sk');
               geo_filter.defineData('geo_hier_sk');
               geo_filter.defineDone();
            %end;
            %if (&contains_prod_hier_sk = 1 and &prod_hier_sk_true = 1) %then %do;
               declare hash prod_filter(hashexp: 5, dataset:"&prod_filter_table");
               prod_filter.defineKey('prod_hier_sk');
               prod_filter.defineData('prod_hier_sk');
               prod_filter.defineDone();
            %end;
            end;
            &filter;
         run;

         %if %di_util_nobs(work._&table_name) gt 0 %then %do;

            %di_util_truncate_table(&purge_lib..&table_name); 

            proc append data=work._&table_name 
            base=&purge_lib..&table_name  force; 
            run; 

            proc datasets lib=work nolist nowarn memtype = (data view);
            delete _&table_name ;
            quit;

         %end;

         %if &table_name=PRODUCT_HIER_ASSOC_DM and &purge_lib=DI_DM_P %then %do;
              %di_prod_delete_dangling_nodes(purge_lib = &purge_lib,
                                                memname = &table_name);
         %end;

         %if &table_name=GEOGRAPHY_HIER_ASSOC_DM and &purge_lib=DI_DM_P %then %do;
              %di_geo_delete_dangling_nodes(purge_lib = &purge_lib,
                                               memname= &table_name);
         %end;

         %put ______: syserr = &syserr;
         
      %end; /*%if &proceed eq 1 %then %do;*/

      %let final_count = %di_util_nobs(&purge_lib..&table_name);
 
      %end; /*end: If the table to be purged belongs to SAS*/
      /*If the table to be purged belongs to RDBMS*/
      %else %if %upcase(&ds_type)=%upcase(RDBMS) %then %do; 
       
         %let purge_lib=DI_DM2;
         proc sql noprint;
            select count(*) into :init_count from &purge_lib..&table_name;
         quit;

         * check for PROD_HIER_SK match;
         %if (&contains_prod_hier_sk = 1 and &prod_hier_sk_true = 1) %then %do;
            %put ______: Purging RDBMS table - &table_name by prod_hier_sk......;     
            %di_dm_trans_delete(di_trans=&purge_lib,
                            trans_table=&table_name,
                            delete_join_table=&prod_filter_table,
                            delete_join_cond=%str(a.prod_hier_sk=b.prod_hier_sk),
                            chunk_size_val=0,
                            chunk = 1 );
         %end;

         * check for GEO_HIER_SK match;
         %if (&contains_geo_hier_sk = 1 and &geo_hier_sk_true = 1) %then %do;
            %put ______: Purging RDBMS table - &table_name by geo_hier_sk ......;   
            %di_dm_trans_delete(di_trans=&purge_lib,
                            trans_table=&table_name,
                            delete_join_table=&geo_filter_table,
                            delete_join_cond=%str(a.geo_hier_sk=b.geo_hier_sk),
                            chunk_size_val=0,
                            chunk = 1 ); 
         %end;

         proc sql noprint;
            select count(*) into :final_count from &purge_lib..&table_name;
         quit;
      %end; /*End:If the table to be purged belongs to RDBMS*/

   %if &syscc > 4 %then %do;
      %let l_purged_flg='N';
      %let syscc=0;
   %end;
   %else %do;
      %let l_purged_flg='Y';
      %let syscc=0;
   %end;

   /*Update di_dm.purge_result_table only when some data was purged from the dataset*/
   %if &init_count ne &final_count %then %do;

      %if &table_name eq PRF_SALES_FACT or  &table_name eq PRF_STOCKOUT_FACT or &table_name eq PRF_GEO_PROD_STATUS or &table_name eq PRF_INVENTORY_FACT %then %do; /* Begin:Loop5*/
            /* This macro purges the deleted records from PRF_* tables */
	        %put ______: Purging deleted records from &purge_lib..&table_name;
            %di_util_purge_deleted_records (&purge_lib..&table_name);

      /*============================================================================================*/
      /* Update the META tables of PRF_* fact tables. The logic is to delete the entries from       */   
      /* META tables for the part_sk in process and re-create the meta information based on the     */ 
      /*latest data available in corresponding fact tables after purging.                           */
      /* ===========================================================================================*/

            %let hx_part_sk=;
            %let orig_part_sk=;
            %let hx_part_sk = %scan(&purge_lib,-1,_);
            %let orig_part_sk = %sysfunc(inputn(&hx_part_sk,hex6.));
            %put orig_part_sk = &orig_part_sk;

            data work.map (keep = PART_SK PROD_HIER_SK START_ROW END_ROW);
            set &purge_lib..&table_name;
            attrib part_sk length = 6;
            attrib START_ROW length = 6;
            attrib END_ROW length = 6;
            PART_SK = &orig_part_sk;
            by PROD_HIER_SK;
            retain START_ROW;
            if first.PROD_HIER_SK then do;
               START_ROW = _n_;
            end;
            if last.PROD_HIER_SK then do;
               END_ROW = _n_;
               output;
            end;
            run;

			%put ______: Updating di_dm.&table_name._meta;

            %di_util_lock(di_dm.&table_name._meta);

                proc sql noprint;
                   delete from di_dm.&table_name._meta 
                   where part_sk = &orig_part_sk;
                quit;

                proc append data = work.map 
                   base = di_dm.&table_name._meta force;
                run;

             %di_util_unlock(di_dm.&table_name._meta);

      %end;
     
	  /*Macro to generate the name of the purged result table along with short path*/
      %gen_table_nm(memname=&table_name, purge_lib=&purge_lib);

      %put ______: purge_table_name=&purge_table_name;

      proc datasets lib=work nolist nowarn; delete info_tab_&ds_type._&k;quit;
      data work.info_tab_&ds_type._&k;
         attrib table_nm length=$100 before_rows length = 4 after_rows length = 4
         purge_dttm length = 8 format=datetime16. purged_flg length=$1;
         table_nm="&purge_table_name";
         before_rows = &init_count;
         after_rows = &final_count;
         format purge_dttm datetime16.;
         purge_dttm=datetime();
         purged_flg=&l_purged_flg;
      run;
   %end;
   /*re-create di_mon.prod_hier_sk*/
   %if %upcase(&table_name)=PRODUCT_DM and %upcase(&purge_lib)=DI_DM_P %then %do;
      %if &syserr le 6 %then %do;
         proc sql noprint;
            create table di_mon.prod_hier_sk 
            as select prod_id, max(prod_hier_sk) as prod_hier_sk length=6
            from di_dm_p.product_dm
            group by prod_id;
         quit;  
      %end;
   %end;

%end;
%end;

/* Append the purge result information from temp tables to di_dm.purge_result_table */
%put ______: Updating &purge_result_table;

%di_util_lock(&purge_result_table);
   %do k= 1 %to &row_cnt;
      %let etls_tabCheckExist = 0;
      %let etls_tabCheckExist = %eval(%sysfunc(exist(work.info_tab_&ds_type._&k, DATA));
      %if (&etls_tabCheckExist) %then %do;
         proc append base=&purge_result_table
                     data=work.info_tab_&ds_type._&k force;
         run;  

/*	 proc datasets nodetails nolist nowarn lib=work;
            delete info_tab_&ds_type._&k;
         quit;*/
	  %end;
   %end;
%di_util_unlock(&purge_result_table);

%mend purge_ind_tbl;

%macro di_util_purge_unit_by_gp(
                                  table_to_purge=,
                                  purge_lib = ,
                                  geo_filter_table = ,
                                  prod_filter_table = ,
                                  purge_result_table =,
                                  ds_type=,
                                  temp_work_path=,
                                  l_prefix=
                                );
%let table_to_purge=&table_to_purge;
%let purge_lib = &purge_lib;
%let geo_filter_table = &geo_filter_table;
%let prod_filter_table = &prod_filter_table;
%let purge_result_table =&purge_result_table;
%let ds_type=&ds_type;
%let temp_work_path=&temp_work_path;
%let l_prefix=&l_prefix;

%put ______: executing di_util_purge_unit_by_gp;
%put "_______Macro di_util_purge_unit_by_gp is from a new catalog";

 data _null_;
    pid  = &SYSJOBID;
    put '************************************************************************';
    put '*   Attention: The Process ID (PID) of this SAS Session is: ' pid ;
    put '************************************************************************';
    put '_';
 run;
 %global  geo_hier_sk_true prod_hier_sk_true;
 %local goto_errorexit column_list purge_result_table filter init_count final_count table_name
        tref rc nvars var contains_geo_hier_sk contains_prod_hier_sk filter tab_length l_purged_flg;

%let goto_errorexit = 0;
%let geo_hier_sk_true = 0;
%let prod_hier_sk_true = 0;
%let syscc=0;
%let purge_result_table=&purge_result_table;

%put ______: geo_filter_table=&geo_filter_table;
%put ______: prod_filter_table=&prod_filter_table;
%put ______: temp_work_path=&temp_work_path;

libname temp_lib "&temp_work_path";

   /* Check if both the filter tables ar empty */

%if %di_util_nobs(&geo_filter_table) = 0 and
    %di_util_nobs(&prod_filter_table) = 0 %then %do;
    %put WARNING: Both the filter tables are either missing or empty. ;
    %let goto_errorexit = 1;
%end;

%if &goto_errorexit %then %do; 
    %let syscc=9999;
    %goto ERROREXIT;
%end;

   /*=================================================================================
   Set the geo_hier_sk_true to 1 if geo_filter_table is non-empty. This variable helps
   deciding whether to purge a table using geo_hier_sk
   ===================================================================================*/

%if %di_util_nobs(&geo_filter_table) > 0 %then %do;
   %let geo_hier_sk_true = 1;
%end;

   /*=================================================================================
   Set the prod_hier_sk_true to 1 if prod_filter_table is non-empty. This variable helps
   deciding whether to purge a table using prod_hier_sk
   ===================================================================================*/

%if %di_util_nobs(&prod_filter_table) > 0 %then %do;
   %let prod_hier_sk_true = 1;
%end;

%let filter = %str();

%let init_count = 0;
%let final_count = 0;

%purge_ind_tbl(table_to_purge=&table_to_purge,
                  purge_lib = &purge_lib,
                  geo_filter_table = &geo_filter_table,
                  prod_filter_table = &prod_filter_table,
                  purge_result_table =&purge_result_table,
                  ds_type=&ds_type,
                  temp_work_path=&temp_work_path);



%ERROREXIT:

   %if &syscc > 4 %then %do;
      
       %put ______: NOTES: Purge process could not be completed successfully. Some issue while executing di_util_purge_unit_by_gp();
       %let syscc=0;
   %end;
   

%mend di_util_purge_unit_by_gp;
